# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Provides functions and *ITR classes, for writing increment files"""

import Globals, Time, rpath, Rdiff, log, statistics, robust


def Increment(new, mirror, incpref):
	"""Main file incrementing function, returns inc file created

	new is the file on the active partition,
	mirror is the mirrored file from the last backup,
	incpref is the prefix of the increment file.

	This function basically moves the information about the mirror
	file to incpref.

	"""
	log.Log("Incrementing mirror file " + mirror.path, 5)
	if ((new and new.isdir()) or mirror.isdir()) and not incpref.lstat():
		incpref.mkdir()

	if not mirror.lstat(): incrp = makemissing(incpref)
	elif mirror.isdir(): incrp = makedir(mirror, incpref)
	elif new.isreg() and mirror.isreg():
		incrp = makediff(new, mirror, incpref)
	else: incrp = makesnapshot(mirror, incpref)
	statistics.process_increment(incrp)
	return incrp

def makemissing(incpref):
	"""Signify that mirror file was missing"""
	incrp = get_inc(incpref, "missing")
	incrp.touch()
	return incrp

def iscompressed(mirror):
	"""Return true if mirror's increments should be compressed"""
	return (Globals.compression and
			not Globals.no_compression_regexp.match(mirror.path))

def makesnapshot(mirror, incpref):
	"""Copy mirror to incfile, since new is quite different"""
	compress = iscompressed(mirror)
	if compress and mirror.isreg():
		snapshotrp = get_inc(incpref, "snapshot.gz")
	else: snapshotrp = get_inc(incpref, "snapshot")

	if mirror.isspecial(): # check for errors when creating special increments
		eh = robust.get_error_handler("SpecialFileError")
		if robust.check_common_error(eh, rpath.copy_with_attribs,
									 (mirror, snapshotrp, compress)) == 0:
			snapshotrp.setdata()
			if snapshotrp.lstat(): snapshotrp.delete()
			snapshotrp.touch()
	else: rpath.copy_with_attribs(mirror, snapshotrp, compress)
	return snapshotrp

def makediff(new, mirror, incpref):
	"""Make incfile which is a diff new -> mirror"""
	compress = iscompressed(mirror)
	if compress: diff = get_inc(incpref, "diff.gz")
	else:  diff = get_inc(incpref, "diff")

	old_new_perms, old_mirror_perms = (None, None)

	if Globals.process_uid != 0:
		# Check for unreadable files
		if not new.readable():
			old_new_perms = new.getperms()
			new.chmod(0400 | old_new_perms)
		if not mirror.readable():
			old_mirror_perms = mirror.getperms()
			mirror.chmod(0400 | old_mirror_perms)
	
	Rdiff.write_delta(new, mirror, diff, compress)

	if old_new_perms: new.chmod(old_new_perms)
	if old_mirror_perms: mirror.chmod(old_mirror_perms)

	rpath.copy_attribs_inc(mirror, diff)
	return diff

def makedir(mirrordir, incpref):
	"""Make file indicating directory mirrordir has changed"""
	dirsign = get_inc(incpref, "dir")
	dirsign.touch()
	rpath.copy_attribs_inc(mirrordir, dirsign)
	return dirsign

def get_inc(rp, typestr, time = None):
	"""Return increment like rp but with time and typestr suffixes

	To avoid any quoting, the returned rpath has empty index, and the
	whole filename is in the base (which is not quoted).

	"""
	if time is None: time = Time.prevtime
	addtostr = lambda s: "%s.%s.%s" % (s, Time.timetostring(time), typestr)
	if rp.index:
		incrp = rp.__class__(rp.conn, rp.base, rp.index[:-1] +
							 (addtostr(rp.index[-1]),))
	else:
		dirname, basename = rp.dirsplit()
		incrp = rp.__class__(rp.conn, dirname, (addtostr(basename),))
	assert not incrp.lstat(), incrp
	return incrp





# Copyright 2002, 2003 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Parse args and setup connections

The functions in this module are used once by Main to parse file
descriptions like bescoto@folly.stanford.edu:/usr/bin/ls and to set up
the related connections.

"""

import os, sys
from log import Log
import Globals, connection, rpath

# This is the schema that determines how rdiff-backup will open a
# pipe to the remote system.  If the file is given as A::B, %s will
# be substituted with A in the schema.
__cmd_schema = 'ssh -C %s rdiff-backup --server'
__cmd_schema_no_compress = 'ssh %s rdiff-backup --server'

# This is a list of remote commands used to start the connections.
# The first is None because it is the local connection.
__conn_remote_cmds = [None]

class SetConnectionsException(Exception): pass

def get_cmd_pairs(arglist, remote_schema = None, remote_cmd = None):
	"""Map the given file descriptions into command pairs

	Command pairs are tuples cmdpair with length 2.  cmdpair[0] is
	None iff it describes a local path, and cmdpair[1] is the path.

	"""
	global __cmd_schema
	if remote_schema: __cmd_schema = remote_schema
	elif not Globals.ssh_compression: __cmd_schema = __cmd_schema_no_compress

	if Globals.remote_tempdir:
		__cmd_schema += (" --tempdir=" + Globals.remote_tempdir)

	if not arglist: return []
	desc_pairs = map(parse_file_desc, arglist)

	if filter(lambda x: x[0], desc_pairs): # True if any host_info found
		if remote_cmd:
			Log.FatalError("The --remote-cmd flag is not compatible "
						   "with remote file descriptions.")
	elif remote_schema:
		Log("Remote schema option ignored - no remote file "
			"descriptions.", 2)
	cmdpairs = map(desc2cmd_pairs, desc_pairs)
	if remote_cmd: # last file description gets remote_cmd
		cmd_pairs[-1] = (remote_cmd, cmd_pairs[-1][1])
	return cmdpairs

def cmdpair2rp(cmd_pair):
	"""Return normalized RPath from cmd_pair (remote_cmd, filename)"""
	cmd, filename = cmd_pair
	if cmd: conn = init_connection(cmd)
	else: conn = Globals.local_connection
	return rpath.RPath(conn, filename).normalize()

def desc2cmd_pairs(desc_pair):
	"""Return pair (remote_cmd, filename) from desc_pair"""
	host_info, filename = desc_pair
	if not host_info: return (None, filename)
	else: return (fill_schema(host_info), filename)

def parse_file_desc(file_desc):
	"""Parse file description returning pair (host_info, filename)

	In other words, bescoto@folly.stanford.edu::/usr/bin/ls =>
	("bescoto@folly.stanford.edu", "/usr/bin/ls").  The
	complication is to allow for quoting of : by a \.  If the
	string is not separated by :, then the host_info is None.

	"""
	def check_len(i):
		if i >= len(file_desc):
			raise SetConnectionsException(
				"Unexpected end to file description %s" % file_desc)

	host_info_list, i, last_was_quoted = [], 0, None
	while 1:
		if i == len(file_desc):
			return (None, file_desc)

		if file_desc[i] == '\\':
			i = i+1
			check_len(i)
			last_was_quoted = 1
		elif (file_desc[i] == ":" and i > 0 and file_desc[i-1] == ":"
			  and not last_was_quoted):
			host_info_list.pop() # Remove last colon from name
			break
		else: last_was_quoted = None
		host_info_list.append(file_desc[i])
		i = i+1

	check_len(i+1)
	return ("".join(host_info_list), file_desc[i+1:])

def fill_schema(host_info):
	"""Fills host_info into the schema and returns remote command"""
	try:
		return __cmd_schema % host_info
	except TypeError:
		Log.FatalError("Invalid remote schema:\n\n%s\n" % __cmd_schema)

def init_connection(remote_cmd):
	"""Run remote_cmd, register connection, and then return it

	If remote_cmd is None, then the local connection will be
	returned.  This also updates some settings on the remote side,
	like global settings, its connection number, and verbosity.

	"""
	if not remote_cmd: return Globals.local_connection

	Log("Executing " + remote_cmd, 4)
	if os.name == "nt":
		import subprocess
		try:
			process = subprocess.Popen(remote_cmd, shell=False, bufsize=0,
								stdin=subprocess.PIPE, 
								stdout=subprocess.PIPE)
			(stdin, stdout) = (process.stdin, process.stdout)
		except OSError:
			(stdin, stdout) = (None, None)
	else:
		stdin, stdout = os.popen2(remote_cmd)
	conn_number = len(Globals.connections)
	conn = connection.PipeConnection(stdout, stdin, conn_number)

	check_connection_version(conn, remote_cmd)
	Log("Registering connection %d" % conn_number, 7)
	init_connection_routing(conn, conn_number, remote_cmd)
	init_connection_settings(conn)
	return conn

def check_connection_version(conn, remote_cmd):
	"""Log warning if connection has different version"""
	try: remote_version = conn.Globals.get('version')
	except connection.ConnectionError, exception:
		Log.FatalError("""%s

Couldn't start up the remote connection by executing

    %s

Remember that, under the default settings, rdiff-backup must be
installed in the PATH on the remote system.  See the man page for more
information on this.  This message may also be displayed if the remote
version of rdiff-backup is quite different from the local version (%s)."""
					   % (exception, remote_cmd, Globals.version))
	except OverflowError, exc:
		Log.FatalError("""Integer overflow while attempting to establish the 
remote connection by executing

    %s

Please make sure that nothing is printed (e.g., by your login shell) when this
command executes. Try running this command:

    %s
	
which should only print out the text: rdiff-backup <version>""" % (remote_cmd,
	remote_cmd.replace("--server", "--version")))
		
	if remote_version != Globals.version:
		Log("Warning: Local version %s does not match remote version %s."
			% (Globals.version, remote_version), 2)

def init_connection_routing(conn, conn_number, remote_cmd):
	"""Called by init_connection, establish routing, conn dict"""
	Globals.connection_dict[conn_number] = conn

	conn.SetConnections.init_connection_remote(conn_number)
	for other_remote_conn in Globals.connections[1:]:
		conn.SetConnections.add_redirected_conn(
			other_remote_conn.conn_number)
		other_remote_conn.SetConnections.add_redirected_conn(conn_number)

	Globals.connections.append(conn)
	__conn_remote_cmds.append(remote_cmd)

def init_connection_settings(conn):
	"""Tell new conn about log settings and updated globals"""
	conn.log.Log.setverbosity(Log.verbosity)
	conn.log.Log.setterm_verbosity(Log.term_verbosity)
	for setting_name in Globals.changed_settings:
		conn.Globals.set(setting_name, Globals.get(setting_name))

def init_connection_remote(conn_number):
	"""Run on server side to tell self that have given conn_number"""
	Globals.connection_number = conn_number
	Globals.local_connection.conn_number = conn_number
	Globals.connection_dict[0] = Globals.connections[1]
	Globals.connection_dict[conn_number] = Globals.local_connection

def add_redirected_conn(conn_number):
	"""Run on server side - tell about redirected connection"""
	Globals.connection_dict[conn_number] = \
		   connection.RedirectedConnection(conn_number)

def UpdateGlobal(setting_name, val):
	"""Update value of global variable across all connections"""
	for conn in Globals.connections:
		conn.Globals.set(setting_name, val)

def BackupInitConnections(reading_conn, writing_conn):
	"""Backup specific connection initialization"""
	reading_conn.Globals.set("isbackup_reader", 1)
	writing_conn.Globals.set("isbackup_writer", 1)
	UpdateGlobal("backup_reader", reading_conn)
	UpdateGlobal("backup_writer", writing_conn)

def CloseConnections():
	"""Close all connections.  Run by client"""
	assert not Globals.server
	for conn in Globals.connections: conn.quit()
	del Globals.connections[1:] # Only leave local connection
	Globals.connection_dict = {0: Globals.local_connection}
	Globals.backup_reader = Globals.isbackup_reader = \
		  Globals.backup_writer = Globals.isbackup_writer = None

def TestConnections():
	"""Test connections, printing results"""
	if len(Globals.connections) == 1: print "No remote connections specified"
	else:
		for i in range(1, len(Globals.connections)): test_connection(i)

def test_connection(conn_number):
	"""Test connection.  conn_number 0 is the local connection"""
	print "Testing server started by: ", __conn_remote_cmds[conn_number]
	conn = Globals.connections[conn_number]
	try:
		assert conn.Globals.get('current_time') is None
		assert type(conn.os.listdir('.')) is list
		version = conn.Globals.get('version')
	except:
		sys.stderr.write("Server tests failed\n")
		raise
	if not version == Globals.version:
		print """Server may work, but there is a version mismatch:
Local version: %s
Remote version: %s

In general, an attempt is made to guarantee compatibility only between
different minor versions of the same stable series.  For instance, you
should expect 0.12.4 and 0.12.7 to be compatible, but not 0.12.7
and 0.13.3, nor 0.13.2 and 0.13.4.
""" % (Globals.version, version)
	else: print "Server OK"



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Manage temp files

Earlier this had routines for keeping track of existing tempfiles.
Now we just use normal rpaths instead of the TempFile class.

"""

import os
import Globals, rpath

# To make collisions less likely, this gets put in the file name
# and incremented whenever a new file is requested.
_tfindex = 0

def new(rp_base):
	"""Return new tempfile that isn't in use in same dir as rp_base"""
	return new_in_dir(rp_base.get_parent_rp())

def new_in_dir(dir_rp):
	"""Return new temp rpath in directory dir_rp"""
	global _tfindex
	assert dir_rp.conn is Globals.local_connection
	while 1:
		if _tfindex > 100000000:
			Log("Warning: Resetting tempfile index", 2)
			_tfindex = 0
		tf = dir_rp.append('rdiff-backup.tmp.%d' % _tfindex)
		_tfindex = _tfindex+1
		if not tf.lstat(): return tf




class TempFile(rpath.RPath):
	"""Like an RPath, but keep track of which ones are still here"""
	def rename(self, rp_dest):
		"""Rename temp file to permanent location, possibly overwriting"""
		if not self.lstat(): # "Moving" empty file, so just delete
			if rp_dest.lstat(): rp_dest.delete()
			remove_listing(self)
			return

		if self.isdir() and not rp_dest.isdir():
			# Cannot move a directory directly over another file
			rp_dest.delete()
		rpath.rename(self, rp_dest)

		# Sometimes this just seems to fail silently, as in one
		# hardlinked twin is moved over the other.  So check to make
		# sure below.
		self.setdata()
		if self.lstat():
			rp_dest.delete()
			rpath.rename(self, rp_dest)
			self.setdata()
			if self.lstat(): raise OSError("Cannot rename tmp file correctly")
		remove_listing(self)

	def delete(self):
		rpath.RPath.delete(self)
		remove_listing(self)




# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""list, delete, and otherwise manage increments"""

from __future__ import generators
from log import Log
import Globals, Time, static, statistics, restore, selection, FilenameMapping


class ManageException(Exception): pass

def get_file_type(rp):
	"""Returns one of "regular", "directory", "missing", or "special"."""
	if not rp.lstat(): return "missing"
	elif rp.isdir(): return "directory"
	elif rp.isreg(): return "regular"
	else: return "special"

def get_inc_type(inc):
	"""Return file type increment represents"""
	assert inc.isincfile()
	type = inc.getinctype()
	if type == "dir": return "directory"
	elif type == "diff": return "regular"
	elif type == "missing": return "missing"
	elif type == "snapshot": return get_file_type(inc)
	else: assert None, "Unknown type %s" % (type,)

def describe_incs_parsable(incs, mirror_time, mirrorrp):
	"""Return a string parsable by computer describing the increments

	Each line is a time in seconds of the increment, and then the
	type of the file.  It will be sorted oldest to newest.  For example:

	10000 regular
	20000 directory
	30000 special
	40000 missing
	50000 regular    <- last will be the current mirror

	"""
	incpairs = [(inc.getinctime(), inc) for inc in incs]
	incpairs.sort()
	result = ["%s %s" % (time, get_inc_type(inc)) for time, inc in incpairs]
	result.append("%s %s" % (mirror_time, get_file_type(mirrorrp)))
	return "\n".join(result)

def describe_incs_human(incs, mirror_time, mirrorrp):
	"""Return a string describing all the the root increments"""
	incpairs = [(inc.getinctime(), inc) for inc in incs]
	incpairs.sort()

	result = ["Found %d increments:" % len(incpairs)]
	if Globals.chars_to_quote:
		for time, inc in incpairs:
			result.append("    %s   %s" %
						  (FilenameMapping.unquote(inc.dirsplit()[1]),
						   Time.timetopretty(time)))
	else:
		for time, inc in incpairs:
			result.append("    %s   %s" %
						  (inc.dirsplit()[1], Time.timetopretty(time)))
	result.append("Current mirror: %s" % Time.timetopretty(mirror_time))
	return "\n".join(result)

def delete_earlier_than(baserp, time):
	"""Deleting increments older than time in directory baserp

	time is in seconds.  It will then delete any empty directories
	in the tree.  To process the entire backup area, the
	rdiff-backup-data directory should be the root of the tree.

	"""
	baserp.conn.manage.delete_earlier_than_local(baserp, time)

def delete_earlier_than_local(baserp, time):
	"""Like delete_earlier_than, but run on local connection for speed"""
	assert baserp.conn is Globals.local_connection
	def yield_files(rp):
		if rp.isdir():
			for filename in rp.listdir():
				for sub_rp in yield_files(rp.append(filename)):
					yield sub_rp
		yield rp

	for rp in yield_files(baserp):
		if ((rp.isincfile() and rp.getinctime() < time) or
			(rp.isdir() and not rp.listdir())):
			Log("Deleting increment file %s" % rp.path, 5)
			rp.delete()


class IncObj:
	"""Increment object - represent a completed increment"""
	def __init__(self, incrp):
		"""IncObj initializer

		incrp is an RPath of a path like increments.TIMESTR.dir
		standing for the root of the increment.

		"""
		if not incrp.isincfile():
			raise ManageException("%s is not an inc file" % incrp.path)
		self.incrp = incrp
		self.time = incrp.getinctime()

	def getbaserp(self):
		"""Return rp of the incrp without extensions"""
		return self.incrp.getincbase()

	def pretty_time(self):
		"""Return a formatted version of inc's time"""
		return Time.timetopretty(self.time)

	def full_description(self):
		"""Return string describing increment"""
		s = ["Increment file %s" % self.incrp.path,
			 "Date: %s" % self.pretty_time()]
		return "\n".join(s)


def ListIncrementSizes(mirror_root, index):
	"""Return string summarizing the size of all the increments"""
	stat_obj = statistics.StatsObj() # used for byte summary string
	def get_total(rp_iter):
		"""Return the total size of everything in rp_iter"""
		total = 0
		for rp in rp_iter: total += rp.getsize()
		return total

	def get_time_dict(inc_iter):
		"""Return dictionary pairing times to total size of incs"""
		time_dict = {}
		for inc in inc_iter:
			if not inc.isincfile(): continue
			t = inc.getinctime()
			if not time_dict.has_key(t): time_dict[t] = 0
			time_dict[t] += inc.getsize()
		return time_dict

	def get_mirror_select():
		"""Return iterator of mirror rpaths"""
		mirror_base = mirror_root.new_index(index)
		mirror_select = selection.Select(mirror_base)
		if not index: # must exclude rdiff-backup-directory
			mirror_select.parse_rbdir_exclude()
		return mirror_select.set_iter()

	def get_inc_select():
		"""Return iterator of increment rpaths"""
		inc_base = Globals.rbdir.append_path('increments', index)
		for base_inc in restore.get_inclist(inc_base): yield base_inc
		if inc_base.isdir():
			inc_select = selection.Select(inc_base).set_iter()
			for inc in inc_select: yield inc

	def get_summary_triples(mirror_total, time_dict):
		"""Return list of triples (time, size, cumulative size)"""
		triples = []

		cur_mir_base = Globals.rbdir.append('current_mirror')
		mirror_time = restore.get_inclist(cur_mir_base)[0].getinctime()
		triples.append((mirror_time, mirror_total, mirror_total))

		inc_times = time_dict.keys()
		inc_times.sort()
		inc_times.reverse()
		cumulative_size = mirror_total
		for inc_time in inc_times:
			size = time_dict[inc_time]
			cumulative_size += size
			triples.append((inc_time, size, cumulative_size))
		return triples

	def triple_to_line(triple):
		"""Convert triple to display string"""
		time, size, cum_size = triple
		return "%24s   %13s   %15s" % \
			   (Time.timetopretty(time),
				stat_obj.get_byte_summary_string(size),
				stat_obj.get_byte_summary_string(cum_size))

	mirror_total = get_total(get_mirror_select())
	time_dict = get_time_dict(get_inc_select())
	triples = get_summary_triples(mirror_total, time_dict)

	l = ['%12s %9s  %15s   %20s' % ('Time', '', 'Size', 'Cumulative size'),
		 '-' * 77,
		 triple_to_line(triples[0]) + '   (current mirror)']
	for triple in triples[1:]: l.append(triple_to_line(triple))
	return '\n'.join(l)

		



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Generate and process aggregated backup information"""

import re, os, time
import Globals, Time, increment, log, static, metadata

class StatsException(Exception): pass

class StatsObj:
	"""Contains various statistics, provide string conversion functions"""
	# used when quoting files in get_stats_line
	space_regex = re.compile(" ")

	stat_file_attrs = ('SourceFiles', 'SourceFileSize',
					   'MirrorFiles', 'MirrorFileSize',
					   'NewFiles', 'NewFileSize',
					   'DeletedFiles', 'DeletedFileSize',
					   'ChangedFiles',
					   'ChangedSourceSize', 'ChangedMirrorSize',
					   'IncrementFiles', 'IncrementFileSize')
	stat_misc_attrs = ('Errors', 'TotalDestinationSizeChange')
	stat_time_attrs = ('StartTime', 'EndTime', 'ElapsedTime')
	stat_attrs = (('Filename',) + stat_time_attrs +
				  stat_misc_attrs + stat_file_attrs)

	# Below, the second value in each pair is true iff the value
	# indicates a number of bytes
	stat_file_pairs = (('SourceFiles', None), ('SourceFileSize', 1),
					   ('MirrorFiles', None), ('MirrorFileSize', 1),
					   ('NewFiles', None), ('NewFileSize', 1),
					   ('DeletedFiles', None), ('DeletedFileSize', 1),
					   ('ChangedFiles', None),
					   ('ChangedSourceSize', 1), ('ChangedMirrorSize', 1),
					   ('IncrementFiles', None), ('IncrementFileSize', 1))

	# This is used in get_byte_summary_string below
	byte_abbrev_list = ((1024*1024*1024*1024, "TB"),
						(1024*1024*1024, "GB"),
						(1024*1024, "MB"),
						(1024, "KB"))

	def __init__(self):
		"""Set attributes to None"""
		for attr in self.stat_attrs: self.__dict__[attr] = None

	def get_stat(self, attribute):
		"""Get a statistic"""
		return self.__dict__[attribute]

	def set_stat(self, attr, value):
		"""Set attribute to given value"""
		self.__dict__[attr] = value

	def increment_stat(self, attr):
		"""Add 1 to value of attribute"""
		self.__dict__[attr] += 1

	def add_to_stat(self, attr, value):
		"""Add value to given attribute"""
		self.__dict__[attr] += value

	def get_total_dest_size_change(self):
		"""Return total destination size change

		This represents the total change in the size of the
		rdiff-backup destination directory.

		"""
		addvals = [self.NewFileSize, self.ChangedSourceSize,
				   self.IncrementFileSize]
		subtractvals = [self.DeletedFileSize, self.ChangedMirrorSize]
		for val in addvals + subtractvals:
			if val is None:
				result = None
				break
		else: 
			def addlist(l): return reduce(lambda x,y: x+y, l)
			result = addlist(addvals) - addlist(subtractvals)
		self.TotalDestinationSizeChange = result
		return result

	def get_stats_line(self, index, use_repr = 1):
		"""Return one line abbreviated version of full stats string"""
		file_attrs = map(lambda attr: str(self.get_stat(attr)),
						 self.stat_file_attrs)
		if not index: filename = "."
		else:
			filename = apply(os.path.join, index)
			if use_repr:
				# use repr to quote newlines in relative filename, then
				# take of leading and trailing quote and quote spaces.
				filename = self.space_regex.sub("\\x20", repr(filename)[1:-1])
		return " ".join([filename,] + file_attrs)

	def set_stats_from_line(self, line):
		"""Set statistics from given line"""
		def error(): raise StatsException("Bad line '%s'" % line)
		if line[-1] == "\n": line = line[:-1]
		lineparts = line.split(" ")
		if len(lineparts) < len(stat_file_attrs): error()
		for attr, val_string in zip(stat_file_attrs,
									lineparts[-len(stat_file_attrs):]):
			try: val = long(val_string)
			except ValueError:
				try: val = float(val_string)
				except ValueError: error()
			self.set_stat(attr, val)
		return self

	def get_stats_string(self):
		"""Return extended string printing out statistics"""
		return "%s%s%s" % (self.get_timestats_string(),
						   self.get_filestats_string(),
						   self.get_miscstats_string())

	def get_timestats_string(self):
		"""Return portion of statistics string dealing with time"""
		timelist = []
		if self.StartTime is not None:
			timelist.append("StartTime %.2f (%s)\n" %
						(self.StartTime, Time.timetopretty(self.StartTime)))
		if self.EndTime is not None:
			timelist.append("EndTime %.2f (%s)\n" %
							(self.EndTime, Time.timetopretty(self.EndTime)))
		if self.ElapsedTime or (self.StartTime is not None and
								self.EndTime is not None):
			if self.ElapsedTime is None:
				self.ElapsedTime = self.EndTime - self.StartTime
			timelist.append("ElapsedTime %.2f (%s)\n" %
				   (self.ElapsedTime, Time.inttopretty(self.ElapsedTime)))
		return "".join(timelist)

	def get_filestats_string(self):
		"""Return portion of statistics string about files and bytes"""
		def fileline(stat_file_pair):
			"""Return zero or one line of the string"""
			attr, in_bytes = stat_file_pair
			val = self.get_stat(attr)
			if val is None: return ""
			if in_bytes:
				return "%s %s (%s)\n" % (attr, val,
										 self.get_byte_summary_string(val))
			else: return "%s %s\n" % (attr, val)

		return "".join(map(fileline, self.stat_file_pairs))

	def get_miscstats_string(self):
		"""Return portion of extended stat string about misc attributes"""
		misc_string = ""
		tdsc = self.get_total_dest_size_change()
		if tdsc is not None:
			misc_string += ("TotalDestinationSizeChange %s (%s)\n" %
							(tdsc, self.get_byte_summary_string(tdsc)))
		if self.Errors is not None: misc_string += "Errors %d\n" % self.Errors
		return misc_string

	def get_byte_summary_string(self, byte_count):
		"""Turn byte count into human readable string like "7.23GB" """
		if byte_count < 0:
			sign = "-"
			byte_count = -byte_count
		else: sign = ""

		for abbrev_bytes, abbrev_string in self.byte_abbrev_list:
			if byte_count >= abbrev_bytes:
				# Now get 3 significant figures
				abbrev_count = float(byte_count)/abbrev_bytes
				if abbrev_count >= 100: precision = 0
				elif abbrev_count >= 10: precision = 1
				else: precision = 2
				return "%s%%.%df %s" % (sign, precision, abbrev_string) \
					   % (abbrev_count,)
		byte_count = round(byte_count)
		if byte_count == 1: return sign + "1 byte"
		else: return "%s%d bytes" % (sign, byte_count)

	def get_stats_logstring(self, title):
		"""Like get_stats_string, but add header and footer"""
		header = "--------------[ %s ]--------------" % title
		footer = "-" * len(header)
		return "%s\n%s%s\n" % (header, self.get_stats_string(), footer)

	def set_stats_from_string(self, s):
		"""Initialize attributes from string, return self for convenience"""
		def error(line): raise StatsException("Bad line '%s'" % line)

		for line in s.split("\n"):
			if not line: continue
			line_parts = line.split()
			if len(line_parts) < 2: error(line)
			attr, value_string = line_parts[:2]
			if not attr in self.stat_attrs: error(line)
			try:
				try: val1 = long(value_string)
				except ValueError: val1 = None
				val2 = float(value_string)
				if val1 == val2: self.set_stat(attr, val1) # use integer val
				else: self.set_stat(attr, val2) # use float
			except ValueError: error(line)
		return self

	def write_stats_to_rp(self, rp):
		"""Write statistics string to given rpath"""
		fp = rp.open("wb")
		fp.write(self.get_stats_string())
		assert not fp.close()

	def read_stats_from_rp(self, rp):
		"""Set statistics from rpath, return self for convenience"""
		fp = rp.open("r")
		self.set_stats_from_string(fp.read())
		fp.close()
		return self

	def stats_equal(self, s):
		"""Return true if s has same statistics as self"""
		assert isinstance(s, StatsObj)
		for attr in self.stat_file_attrs:
			if self.get_stat(attr) != s.get_stat(attr): return None
		return 1

	def set_to_average(self, statobj_list):
		"""Set self's attributes to average of those in statobj_list"""
		for attr in self.stat_attrs: self.set_stat(attr, 0)
		for statobj in statobj_list:
			for attr in self.stat_attrs:
				if statobj.get_stat(attr) is None: self.set_stat(attr, None)
				elif self.get_stat(attr) is not None:
					self.set_stat(attr, statobj.get_stat(attr) +
								  self.get_stat(attr))

		# Don't compute average starting/stopping time		
		self.StartTime = None
		self.EndTime = None

		for attr in self.stat_attrs:
			if self.get_stat(attr) is not None:
				self.set_stat(attr,
							  self.get_stat(attr)/float(len(statobj_list)))
		return self

	def get_statsobj_copy(self):
		"""Return new StatsObj object with same stats as self"""
		s = StatObj()
		for attr in self.stat_attrs: s.set_stat(attr, self.get_stat(attr))
		return s


class StatFileObj(StatsObj):
	"""Build on StatsObj, add functions for processing files"""
	def __init__(self, start_time = None):
		"""StatFileObj initializer - zero out file attributes"""
		StatsObj.__init__(self)
		for attr in self.stat_file_attrs: self.set_stat(attr, 0)
		if start_time is None: start_time = Time.curtime
		self.StartTime = start_time
		self.Errors = 0

	def add_source_file(self, src_rorp):
		"""Add stats of source file"""
		self.SourceFiles += 1
		if src_rorp.isreg(): self.SourceFileSize += src_rorp.getsize()

	def add_dest_file(self, dest_rorp):
		"""Add stats of destination size"""
		self.MirrorFiles += 1
		if dest_rorp.isreg(): self.MirrorFileSize += dest_rorp.getsize()

	def add_changed(self, src_rorp, dest_rorp):
		"""Update stats when src_rorp changes to dest_rorp"""
		if src_rorp and src_rorp.lstat() and dest_rorp and dest_rorp.lstat():
			self.ChangedFiles += 1
			if src_rorp.isreg(): self.ChangedSourceSize += src_rorp.getsize()
			if dest_rorp.isreg(): self.ChangedMirrorSize += dest_rorp.getsize()
		elif src_rorp and src_rorp.lstat():
			self.NewFiles += 1
			if src_rorp.isreg(): self.NewFileSize += src_rorp.getsize()
		elif dest_rorp and dest_rorp.lstat():
			self.DeletedFiles += 1
			if dest_rorp.isreg(): self.DeletedFileSize += dest_rorp.getsize()

	def add_increment(self, inc_rorp):
		"""Update stats with increment rorp"""
		self.IncrementFiles += 1
		if inc_rorp.isreg(): self.IncrementFileSize += inc_rorp.getsize()
		
	def add_error(self):
		"""Increment error stat by 1"""
		self.Errors += 1

	def finish(self, end_time = None):
		"""Record end time and set other stats"""
		if end_time is None: end_time = time.time()
		self.EndTime = end_time


_active_statfileobj = None
def init_statfileobj():
	"""Return new stat file object, record as active stat object"""
	global _active_statfileobj
	assert not _active_statfileobj, _active_statfileobj
	_active_statfileobj = StatFileObj()
	return _active_statfileobj

def get_active_statfileobj():
	"""Return active stat file object if it exists"""
	if _active_statfileobj: return _active_statfileobj
	else: return None

def record_error():
	"""Record error on active statfileobj, if there is one"""
	if _active_statfileobj: _active_statfileobj.add_error()

def process_increment(inc_rorp):
	"""Add statistics of increment rp incrp if there is active statfile"""
	if _active_statfileobj: _active_statfileobj.add_increment(inc_rorp)

def write_active_statfileobj():
	"""Write active StatFileObj object to session statistics file"""
	global _active_statfileobj
	assert _active_statfileobj
	rp_base = Globals.rbdir.append("session_statistics")
	session_stats_rp = increment.get_inc(rp_base, 'data', Time.curtime)
	_active_statfileobj.finish()
	_active_statfileobj.write_stats_to_rp(session_stats_rp)
	_active_statfileobj = None

def print_active_stats():
	"""Print statistics of active statobj to stdout and log"""
	global _active_statfileobj
	assert _active_statfileobj
	_active_statfileobj.finish()
	statmsg = _active_statfileobj.get_stats_logstring("Session statistics")
	log.Log.log_to_file(statmsg)
	Globals.client_conn.sys.stdout.write(statmsg)


class FileStats:
	"""Keep track of less detailed stats on file-by-file basis"""
	_fileobj, _rp = None, None
	_line_sep = None
	def init(cls):
		"""Open file stats object and prepare to write"""
		assert not (cls._fileobj or cls._rp), (cls._fileobj, cls._rp)
		rpbase = Globals.rbdir.append("file_statistics")
		suffix = Globals.compression and 'data.gz' or 'data'
		cls._rp = increment.get_inc(rpbase, suffix, Time.curtime)
		assert not cls._rp.lstat()
		cls._fileobj = cls._rp.open("wb", compress = Globals.compression)

		cls._line_sep = Globals.null_separator and '\0' or '\n'
		cls.write_docstring()
		cls.line_buffer = []

	def write_docstring(cls):
		"""Write the first line (a documentation string) into file"""
		cls._fileobj.write("# Format of each line in file statistics file:")
		cls._fileobj.write(cls._line_sep)
		cls._fileobj.write("# Filename Changed SourceSize MirrorSize "
						   "IncrementSize" + cls._line_sep)

	def update(cls, source_rorp, dest_rorp, changed, inc):
		"""Update file stats with given information"""
		if source_rorp: filename = source_rorp.get_indexpath()
		else: filename = dest_rorp.get_indexpath()
		filename = metadata.quote_path(filename)

		size_list = map(cls.get_size, [source_rorp, dest_rorp, inc])
		line = " ".join([filename, str(changed)] + size_list)
		cls.line_buffer.append(line)
		if len(cls.line_buffer) >= 100: cls.write_buffer()

	def get_size(cls, rorp):
		"""Return the size of rorp as string, or "NA" if not a regular file"""
		if not rorp: return "NA"
		if rorp.isreg(): return str(rorp.getsize())
		else: return "0"

	def write_buffer(cls):
		"""Write buffer to file because buffer is full

		The buffer part is necessary because the GzipFile.write()
		method seems fairly slow.

		"""
		assert cls.line_buffer and cls._fileobj
		cls.line_buffer.append('') # have join add _line_sep to end also
		cls._fileobj.write(cls._line_sep.join(cls.line_buffer))
		cls.line_buffer = []

	def close(cls):
		"""Close file stats file"""
		assert cls._fileobj, cls._fileobj
		if cls.line_buffer: cls.write_buffer()
		assert not cls._fileobj.close()
		cls._fileobj = cls._rp = None

static.MakeClass(FileStats)



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Provide time related exceptions and functions"""

import time, types, re, sys, calendar
import Globals


class TimeException(Exception): pass

_interval_conv_dict = {"s": 1, "m": 60, "h": 3600, "D": 86400,
					   "W": 7*86400, "M": 30*86400, "Y": 365*86400}
_integer_regexp = re.compile("^[0-9]+$")
_session_regexp = re.compile("^[0-9]+B$")
_interval_regexp = re.compile("^([0-9]+)([smhDWMY])")
_genstr_date_regexp1 = re.compile("^(?P<year>[0-9]{4})[-/]"
					   "(?P<month>[0-9]{1,2})[-/](?P<day>[0-9]{1,2})$")
_genstr_date_regexp2 = re.compile("^(?P<month>[0-9]{1,2})[-/]"
					   "(?P<day>[0-9]{1,2})[-/](?P<year>[0-9]{4})$")
curtime = curtimestr = None

def setcurtime(curtime = None):
	"""Sets the current time in curtime and curtimestr on all systems"""
	t = curtime or time.time()
	for conn in Globals.connections:
		conn.Time.setcurtime_local(long(t))

def setcurtime_local(timeinseconds):
	"""Only set the current time locally"""
	global curtime, curtimestr
	curtime, curtimestr = timeinseconds, timetostring(timeinseconds)

def setprevtime(timeinseconds):
	"""Sets the previous inc time in prevtime and prevtimestr"""
	assert 0 < timeinseconds < curtime, \
		   "Time %s is out of bounds" % (timeinseconds,)
	timestr = timetostring(timeinseconds)
	for conn in Globals.connections:
		conn.Time.setprevtime_local(timeinseconds, timestr)

def setprevtime_local(timeinseconds, timestr):
	"""Like setprevtime but only set the local version"""
	global prevtime, prevtimestr
	prevtime, prevtimestr = timeinseconds, timestr

def timetostring(timeinseconds):
	"""Return w3 datetime compliant listing of timeinseconds"""
	s = time.strftime("%Y-%m-%dT%H:%M:%S", time.localtime(timeinseconds))
	return s + gettzd(timeinseconds)

def stringtotime(timestring):
	"""Return time in seconds from w3 timestring

	If there is an error parsing the string, or it doesn't look
	like a w3 datetime string, return None.

	"""
	try:
		date, daytime = timestring[:19].split("T")
		year, month, day = map(int, date.split("-"))
		hour, minute, second = map(int, daytime.split(":"))
		assert 1900 < year < 2100, year
		assert 1 <= month <= 12
		assert 1 <= day <= 31
		assert 0 <= hour <= 23
		assert 0 <= minute <= 59
		assert 0 <= second <= 61  # leap seconds
		timetuple = (year, month, day, hour, minute, second, -1, -1, 0)
		utc_in_secs = calendar.timegm(timetuple)

		return long(utc_in_secs) + tzdtoseconds(timestring[19:])
	except (TypeError, ValueError, AssertionError): return None

def timetopretty(timeinseconds):
	"""Return pretty version of time"""
	return time.asctime(time.localtime(timeinseconds))

def stringtopretty(timestring):
	"""Return pretty version of time given w3 time string"""
	return timetopretty(stringtotime(timestring))

def prettytotime(prettystring):
	"""Converts time like "Mon Jun 5 11:00:23" to epoch sec, or None"""
	try: return time.mktime(time.strptime(prettystring))
	except ValueError: return None

def inttopretty(seconds):
	"""Convert num of seconds to readable string like "2 hours"."""
	partlist = []
	hours, seconds = divmod(seconds, 3600)
	if hours > 1: partlist.append("%d hours" % hours)
	elif hours == 1: partlist.append("1 hour")

	minutes, seconds = divmod(seconds, 60)
	if minutes > 1: partlist.append("%d minutes" % minutes)
	elif minutes == 1: partlist.append("1 minute")

	if seconds == 1: partlist.append("1 second")
	elif not partlist or seconds > 1:
		if isinstance(seconds, int) or isinstance(seconds, long):
			partlist.append("%s seconds" % seconds)
		else: partlist.append("%.2f seconds" % seconds)
	return " ".join(partlist)

def intstringtoseconds(interval_string):
	"""Convert a string expressing an interval (e.g. "4D2s") to seconds"""
	def error():
		raise TimeException("""Bad interval string "%s"

Intervals are specified like 2Y (2 years) or 2h30m (2.5 hours).  The
allowed special characters are s, m, h, D, W, M, and Y.  See the man
page for more information.
""" % interval_string)
	if len(interval_string) < 2: error()

	total = 0
	while interval_string:
		match = _interval_regexp.match(interval_string)
		if not match: error()
		num, ext = int(match.group(1)), match.group(2)
		if not ext in _interval_conv_dict or num < 0: error()
		total += num*_interval_conv_dict[ext]
		interval_string = interval_string[match.end(0):]
	return total

def gettzd(timeinseconds = None):
	"""Return w3's timezone identification string.

	Expresed as [+/-]hh:mm.  For instance, PDT is -07:00 during
	dayling savings and -08:00 otherwise.  Zone coincides with what
	localtime(), etc., use.  If no argument given, use the current
	time.

	"""
	if timeinseconds is None: timeinseconds = time.time()
	dst_in_effect = time.daylight and time.localtime(timeinseconds)[8]
	if dst_in_effect: offset = -time.altzone/60
	else: offset = -time.timezone/60
	if offset > 0: prefix = "+"
	elif offset < 0: prefix = "-"
	else: return "Z" # time is already in UTC

	hours, minutes = map(abs, divmod(offset, 60))
	assert 0 <= hours <= 23
	assert 0 <= minutes <= 59
	return "%s%02d:%02d" % (prefix, hours, minutes)

def tzdtoseconds(tzd):
	"""Given w3 compliant TZD, return how far ahead UTC is"""
	if tzd == "Z": return 0
	assert len(tzd) == 6 # only accept forms like +08:00 for now
	assert (tzd[0] == "-" or tzd[0] == "+") and tzd[3] == ":"
	return -60 * (60 * int(tzd[:3]) + int(tzd[4:]))

def cmp(time1, time2):
	"""Compare time1 and time2 and return -1, 0, or 1"""
	if type(time1) is types.StringType:
		time1 = stringtotime(time1)
		assert time1 is not None
	if type(time2) is types.StringType:
		time2 = stringtotime(time2)
		assert time2 is not None

	if time1 < time2: return -1
	elif time1 == time2: return 0
	else: return 1

def time_from_session(session_num, rp = None):
	"""Return time in seconds of given backup

	The current mirror is session_num 0, the next oldest increment has
	number 1, etc.  Requires that the Globals.rbdir directory be set.

	"""
	session_times = Globals.rbdir.conn.restore.MirrorStruct \
					.get_increment_times()
	session_times.sort()
	if len(session_times) <= session_num:
		return session_times[0] # Use oldest if too few backups
	return session_times[-session_num-1]
	
def genstrtotime(timestr, curtime = None, rp = None):
	"""Convert a generic time string to a time in seconds

	rp is used when the time is of the form "4B" or similar.  Then the
	times of the increments of that particular file are used.

	"""
	if curtime is None: curtime = globals()['curtime']
	if timestr == "now": return curtime

	def error():
		raise TimeException("""Bad time string "%s"

The acceptible time strings are intervals (like "3D64s"), w3-datetime
strings, like "2002-04-26T04:22:01-07:00" (strings like
"2002-04-26T04:22:01" are also acceptable - rdiff-backup will use the
current time zone), or ordinary dates like 2/4/1997 or 2001-04-23
(various combinations are acceptable, but the month always precedes
the day).""" % timestr)

	# Test for straight integer
	if _integer_regexp.search(timestr): return int(timestr)

	# Test for w3-datetime format, possibly missing tzd
	t = stringtotime(timestr) or stringtotime(timestr+gettzd())
	if t: return t

	# Test for time given as number of backups, like 3B
	if _session_regexp.search(timestr):
		return time_from_session(int(timestr[:-1]), rp)

	# Try for long time, like "Mon Jun 5 11:00:23 1990"
	t = prettytotime(timestr)
	if t is not None: return t

	try: # test for an interval, like "2 days ago"
		return curtime - intstringtoseconds(timestr)
	except TimeException: pass

	# Now check for dates like 2001/3/23
	match = _genstr_date_regexp1.search(timestr) or \
			_genstr_date_regexp2.search(timestr)
	if not match: error()
	timestr = "%s-%02d-%02dT00:00:00%s" % (match.group('year'),
			     int(match.group('month')), int(match.group('day')), gettzd())
	t = stringtotime(timestr)
	if t is not None: return t
	else: error()




# Copyright 2002, 2003 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Coordinate corresponding files with different names

For instance, some source filenames may contain characters not allowed
on the mirror end.  These files must be called something different on
the mirror end, so we escape the offending characters with semicolons.

One problem/complication is that all this escaping may put files over
the 256 or whatever limit on the length of file names.  (We just don't
handle that error.)

"""

from __future__ import generators
import os, re, types
import Globals, log, rpath

# If true, enable character quoting, and set characters making
# regex-style range.
chars_to_quote = None

# These compiled regular expressions are used in quoting and unquoting
chars_to_quote_regexp = None
unquoting_regexp = None

# Use given char to quote.  Default is set in Globals.
quoting_char = None


class QuotingException(Exception): pass

def set_init_quote_vals():
	"""Set quoting value from Globals on all conns"""
	for conn in Globals.connections:
		conn.FilenameMapping.set_init_quote_vals_local()

def set_init_quote_vals_local():
	"""Set value on local connection, initialize regexps"""
	global chars_to_quote, quoting_char
	chars_to_quote = Globals.chars_to_quote
	if len(Globals.quoting_char) != 1:
		log.Log.FatalError("Expected single character for quoting char,"
						   "got '%s' instead" % (Globals.quoting_char,))
	quoting_char = Globals.quoting_char
	init_quoting_regexps()

def init_quoting_regexps():
	"""Compile quoting regular expressions"""
	global chars_to_quote_regexp, unquoting_regexp
	assert chars_to_quote and type(chars_to_quote) is types.StringType, \
		   "Chars to quote: '%s'" % (chars_to_quote,)
	try:
		chars_to_quote_regexp = \
				 re.compile("[%s]|%s" % (chars_to_quote, quoting_char), re.S)
		unquoting_regexp = re.compile("%s[0-9]{3}" % quoting_char, re.S)
	except re.error:
		log.Log.FatalError("Error '%s' when processing char quote list %r" %
						   (re.error, chars_to_quote))

def quote(path):
	"""Return quoted version of given path

	Any characters quoted will be replaced by the quoting char and
	the ascii number of the character.  For instance, "10:11:12"
	would go to "10;05811;05812" if ":" were quoted and ";" were
	the quoting character.

	"""
	QuotedPath = chars_to_quote_regexp.sub(quote_single, path)
	if not Globals.must_escape_dos_devices \
			and not Globals.must_escape_trailing_spaces:
		return QuotedPath

	# Escape a trailing space or period (invalid in names on FAT32 under DOS,
	# Windows and modern Linux)
	if Globals.must_escape_trailing_spaces:
		if len(QuotedPath) and (QuotedPath[-1] == ' ' or QuotedPath[-1] == '.'):
			QuotedPath = QuotedPath[:-1] + \
				"%s%03d" % (quoting_char, ord(QuotedPath[-1]))

		if not Globals.must_escape_dos_devices:
			return QuotedPath

	# Escape first char of any special DOS device files even if filename has an
	# extension.  Special names are: aux, prn, con, nul, com0-9, and lpt1-9.
	if not re.search(r"^aux(\..*)*$|^prn(\..*)*$|^con(\..*)*$|^nul(\..*)*$|" \
					 r"^com[0-9](\..*)*$|^lpt[1-9]{1}(\..*)*$", QuotedPath, \
					 re.I):
		return QuotedPath
	return "%s%03d" % (quoting_char, ord(QuotedPath[0])) + QuotedPath[1:]

def quote_single(match):
	"""Return replacement for a single character"""
	return "%s%03d" % (quoting_char, ord(match.group()))

def unquote(path):
	"""Return original version of quoted filename"""
	return unquoting_regexp.sub(unquote_single, path)

def unquote_single(match):
	"""Unquote a single quoted character"""
	if not len(match.group()) == 4:
		raise QuotingException("Quoted group wrong size: " + match.group())
	try: return chr(int(match.group()[1:]))
	except ValueError:
		raise QuotingException("Quoted out of range: " + match.group())


class QuotedRPath(rpath.RPath):
	"""RPath where the filename is quoted version of index

	We use QuotedRPaths so we don't need to remember to quote RPaths
	derived from this one (via append or new_index).  Note that only
	the index is quoted, not the base.

	"""
	def __init__(self, connection, base, index = (), data = None):
		"""Make new QuotedRPath"""
		self.quoted_index = tuple(map(quote, index))
		self.conn = connection
		self.index = index
		self.base = base
		if base is not None:
			if base == "/": self.path = "/" + "/".join(self.quoted_index)
			else: self.path = "/".join((base,) + self.quoted_index)
		self.file = None
		if data or base is None: self.data = data
		else: self.setdata()

	def __setstate__(self, rpath_state):
		"""Reproduce QuotedRPath from __getstate__ output"""
		conn_number, self.base, self.index, self.data = rpath_state
		self.conn = Globals.connection_dict[conn_number]
		self.quoted_index = tuple(map(quote, self.index))
		self.path = "/".join((self.base,) + self.quoted_index)

	def listdir(self):
		"""Return list of unquoted filenames in current directory

		We want them unquoted so that the results can be sorted
		correctly and append()ed to the currect QuotedRPath.

		"""
		return map(unquote, self.conn.os.listdir(self.path))

	def __str__(self):
		return "QuotedPath: %s\nIndex: %s\nData: %s" % \
			   (self.path, self.index, self.data)

	def isincfile(self):
		"""Return true if path indicates increment, sets various variables"""
		if not self.index: # consider the last component as quoted
			dirname, basename = self.dirsplit()
			temp_rp = rpath.RPath(self.conn, dirname, (unquote(basename),))
			result = temp_rp.isincfile()
			if result:
				self.inc_basestr = unquote(temp_rp.inc_basestr)
				self.inc_timestr = unquote(temp_rp.inc_timestr)
		else:
			result = rpath.RPath.isincfile(self)
			if result: self.inc_basestr = unquote(self.inc_basestr)
		return result
		
def get_quotedrpath(rp, separate_basename = 0):
	"""Return quoted version of rpath rp"""
	assert not rp.index # Why would we starting quoting "in the middle"?
	if separate_basename:
		dirname, basename = rp.dirsplit()
		return QuotedRPath(rp.conn, dirname, (unquote(basename),), rp.data)
	else: return QuotedRPath(rp.conn, rp.base, (), rp.data)

def get_quoted_sep_base(filename):
	"""Get QuotedRPath from filename assuming last bit is quoted"""
	return get_quotedrpath(rpath.RPath(Globals.local_connection, filename), 1)

def update_quoting(rbdir):
	"""Update the quoting of a repository by renaming any
	files that should be quoted differently.
	"""

	def requote(name):
		unquoted_name = unquote(name)
		quoted_name = quote(unquoted_name)
		if name != quoted_name:
			return quoted_name
		else:
			return None
	
	def process(dirpath_rp, name, list):
		new_name = requote(name)
		if new_name:
			if list:
				list.remove(name)
				list.append(new_name)
			name_rp = dirpath_rp.append(name)
			new_rp = dirpath_rp.append(new_name)
			log.Log("Re-quoting %s to %s" % (name_rp.path, new_rp.path), 5)
			rpath.move(name_rp, new_rp)

	assert rbdir.conn is Globals.local_connection
	mirror_rp = rbdir.get_parent_rp()
	mirror = mirror_rp.path

	log.Log("Re-quoting repository %s" % mirror_rp.path, 3)

	try:
		os_walk = os.walk
	except AttributeError:
		os_walk = walk

	for dirpath, dirs, files in os_walk(mirror):
		dirpath_rp = mirror_rp.newpath(dirpath)

		for name in dirs: process(dirpath_rp, name, dirs)
		for name in files: process(dirpath_rp, name, None)

"""
os.walk() copied directly from Python 2.5.1's os.py

Backported here for Python 2.2 support. os.walk() was first added
in Python 2.3.
"""
def walk(top, topdown=True, onerror=None):
	from os import error, listdir
	from os.path import join, isdir, islink
	# We may not have read permission for top, in which case we can't
	# get a list of the files the directory contains.  os.path.walk
	# always suppressed the exception then, rather than blow up for a
	# minor reason when (say) a thousand readable directories are still
	# left to visit.  That logic is copied here.
	try:
		# Note that listdir and error are globals in this module due
		# to earlier import-*.
		names = listdir(top)
	except error, err:
		if onerror is not None:
			onerror(err)
		return

	dirs, nondirs = [], []
	for name in names:
		if isdir(join(top, name)):
			dirs.append(name)
		else:
			nondirs.append(name)

	if topdown:
		yield top, dirs, nondirs
	for name in dirs:
		path = join(top, name)
		if not islink(path):
			for x in walk(path, topdown, onerror):
				yield x
	if not topdown:
		yield top, dirs, nondirs



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Iterate exactly the requested files in a directory

Parses includes and excludes to yield correct files.  More
documentation on what this code does can be found on the man page.

"""

from __future__ import generators
import re
import FilenameMapping, robust, rpath, Globals, log, rorpiter


class SelectError(Exception):
	"""Some error dealing with the Select class"""
	pass

class FilePrefixError(SelectError):
	"""Signals that a specified file doesn't start with correct prefix"""
	pass

class GlobbingError(SelectError):
	"""Something has gone wrong when parsing a glob string"""
	pass


class Select:
	"""Iterate appropriate RPaths in given directory

	This class acts as an iterator on account of its next() method.
	Basically, it just goes through all the files in a directory in
	order (depth-first) and subjects each file to a bunch of tests
	(selection functions) in order.  The first test that includes or
	excludes the file means that the file gets included (iterated) or
	excluded.  The default is include, so with no tests we would just
	iterate all the files in the directory in order.

	The one complication to this is that sometimes we don't know
	whether or not to include a directory until we examine its
	contents.  For instance, if we want to include all the **.py
	files.  If /home/ben/foo.py exists, we should also include /home
	and /home/ben, but if these directories contain no **.py files,
	they shouldn't be included.  For this reason, a test may not
	include or exclude a directory, but merely "scan" it.  If later a
	file in the directory gets included, so does the directory.

	As mentioned above, each test takes the form of a selection
	function.  The selection function takes an rpath, and returns:

	None - means the test has nothing to say about the related file
	0 - the file is excluded by the test
	1 - the file is included
	2 - the test says the file (must be directory) should be scanned

	Also, a selection function f has a variable f.exclude which should
	be true iff f could potentially exclude some file.  This is used
	to signal an error if the last function only includes, which would
	be redundant and presumably isn't what the user intends.

	"""
	# This re should not match normal filenames, but usually just globs
	glob_re = re.compile("(.*[*?[\\\\]|ignorecase\\:)", re.I | re.S)

	def __init__(self, rootrp):
		"""Select initializer.  rpath is the root directory"""
		assert isinstance(rootrp, rpath.RPath)
		self.selection_functions = []
		self.rpath = rootrp
		self.prefix = self.rpath.path

	def set_iter(self, sel_func = None):
		"""Initialize more variables, get ready to iterate

		Selection function sel_func is called on each rpath and is
		usually self.Select.  Returns self just for convenience.

		"""
		if not sel_func: sel_func = self.Select
		self.rpath.setdata() # this may have changed since Select init
		self.iter = self.Iterate_fast(self.rpath, sel_func)
		self.next = self.iter.next
		self.__iter__ = lambda: self
		return self

	def Iterate_fast(self, rpath, sel_func):
		"""Like Iterate, but don't recur, saving time"""
		def error_handler(exc, filename):
			log.ErrorLog.write_if_open("ListError",
									   rpath.index + (filename,), exc)
			return None

		def diryield(rpath):
			"""Generate relevant files in directory rpath

			Returns (rpath, num) where num == 0 means rpath should be
			generated normally, num == 1 means the rpath is a directory
			and should be included iff something inside is included.

			"""
			for filename in self.listdir(rpath):
				new_rpath = robust.check_common_error(error_handler,
										rpath.append, (filename,))
				if new_rpath and new_rpath.lstat():
					s = sel_func(new_rpath)
					if s == 1: yield (new_rpath, 0)
					elif s == 2 and new_rpath.isdir(): yield (new_rpath, 1)

		yield rpath
		if not rpath.isdir(): return
		diryield_stack = [diryield(rpath)]
		delayed_rp_stack = []

		while diryield_stack:
			try: rpath, val = diryield_stack[-1].next()
			except StopIteration:
				diryield_stack.pop()
				if delayed_rp_stack: delayed_rp_stack.pop()
				continue
			if val == 0:
				if delayed_rp_stack:
					for delayed_rp in delayed_rp_stack: yield delayed_rp
					del delayed_rp_stack[:]
				yield rpath
				if rpath.isdir(): diryield_stack.append(diryield(rpath))
			elif val == 1:
				delayed_rp_stack.append(rpath)
				diryield_stack.append(diryield(rpath))

	def Iterate(self, rp, rec_func, sel_func):
		"""Return iterator yielding rpaths in rpath

		rec_func is usually the same as this function and is what
		Iterate uses to find files in subdirectories.  It is used in
		iterate_starting_from.

		sel_func is the selection function to use on the rpaths.  It
		is usually self.Select.

		"""
		s = sel_func(rp)
		if s == 0: return
		elif s == 1: # File is included
			yield rp
			if rp.isdir():
				for rp2 in self.iterate_in_dir(rp, rec_func, sel_func):
					yield rp2
		elif s == 2:
			if rp.isdir(): # Directory is merely scanned
				iid = self.iterate_in_dir(rp, rec_func, sel_func)
				try: first = iid.next()
				except StopIteration: return # no files inside; skip rp
				yield rp
				yield first
				for rp2 in iid: yield rp2
		else: assert 0, "Invalid selection result %s" % (str(s),)

	def listdir(self, dir_rp):
		"""List directory rpath with error logging"""
		def error_handler(exc):
			log.ErrorLog.write_if_open("ListError", dir_rp, exc)
			return []
		dir_listing = robust.check_common_error(error_handler, dir_rp.listdir)
		dir_listing.sort()
		return dir_listing

	def iterate_in_dir(self, rpath, rec_func, sel_func):
		"""Iterate the rpaths in directory rpath."""
		def error_handler(exc, filename):
			log.ErrorLog.write_if_open("ListError",
									   rpath.index + (filename,), exc)
			return None

		for filename in self.listdir(rpath):
			new_rp = robust.check_common_error(
				error_handler, rpath.append, [filename])
			if new_rp:
				for rp in rec_func(new_rp, rec_func, sel_func):
					yield rp

	def FilterIter(self, rorp_iter):
		"""Filter rorp_iter using Select below, removing excluded rorps"""
		def getrpiter(rorp_iter):
			"""Return rp iter by adding indicies of rorp_iter to self.rpath"""
			for rorp in rorp_iter:
				yield rpath.RPath(self.rpath.conn, self.rpath.base,
								  rorp.index, rorp.data)

		ITR = rorpiter.IterTreeReducer(FilterIterITRB, [self])
		for rp in rp_iter: ITR(rp.index, rp)
		ITR.Finish()

	def Select(self, rp):
		"""Run through the selection functions and return dominant val 0/1/2"""
		scanned = 0 # 0, by default, or 2 if prev sel func scanned rp
		for sf in self.selection_functions:
			result = sf(rp)
			if result == 1: return 1
			elif result == 0: return scanned
			elif result == 2: scanned = 2
		return 1

	def ParseArgs(self, argtuples, filelists):
		"""Create selection functions based on list of tuples

		The tuples have the form (option string, additional argument)
		and are created when the initial commandline arguments are
		read.  The reason for the extra level of processing is that
		the filelists may only be openable by the main connection, but
		the selection functions need to be on the backup reader or
		writer side.  When the initial arguments are parsed the right
		information is sent over the link.

		"""
		filelists_index = 0
		try:
			for opt, arg in argtuples:
				if opt == "--exclude":
					self.add_selection_func(self.glob_get_sf(arg, 0))
				elif opt == "--exclude-if-present":
					self.add_selection_func(self.presence_get_sf(arg, 0))
				elif opt == "--exclude-device-files":
					self.add_selection_func(self.devfiles_get_sf(0))
				elif opt == "--exclude-symbolic-links":
					self.add_selection_func(self.symlinks_get_sf(0))
				elif opt == "--exclude-sockets":
					self.add_selection_func(self.sockets_get_sf(0))
				elif opt == "--exclude-fifos":
					self.add_selection_func(self.fifos_get_sf(0))
				elif opt == "--exclude-filelist":
					self.add_selection_func(self.filelist_get_sf(
						filelists[filelists_index], 0, arg))
					filelists_index += 1
				elif opt == "--exclude-globbing-filelist":
					map(self.add_selection_func,
						self.filelist_globbing_get_sfs(
						           filelists[filelists_index], 0, arg))
					filelists_index += 1
				elif opt == "--exclude-other-filesystems":
					self.add_selection_func(self.other_filesystems_get_sf(0))
				elif opt == "--exclude-regexp":
					self.add_selection_func(self.regexp_get_sf(arg, 0))
				elif opt == "--exclude-special-files":
					self.add_selection_func(self.special_get_sf(0))
				elif opt == "--include":
					self.add_selection_func(self.glob_get_sf(arg, 1))
				elif opt == "--include-filelist":
					self.add_selection_func(self.filelist_get_sf(
						filelists[filelists_index], 1, arg))
					filelists_index += 1
				elif opt == "--include-globbing-filelist":
					map(self.add_selection_func,
						self.filelist_globbing_get_sfs(
						          filelists[filelists_index], 1, arg))
					filelists_index += 1
				elif opt == "--include-regexp":
					self.add_selection_func(self.regexp_get_sf(arg, 1))
				elif opt == "--include-special-files":
					self.add_selection_func(self.special_get_sf(1))
				elif opt == "--include-symbolic-links":
					self.add_selection_func(self.symlinks_get_sf(1))
				elif opt == "--max-file-size":
					self.add_selection_func(self.size_get_sf(1, arg))
				elif opt == "--min-file-size":
					self.add_selection_func(self.size_get_sf(0, arg))
				else: assert 0, "Bad selection option %s" % opt
		except SelectError, e: self.parse_catch_error(e)
		assert filelists_index == len(filelists)

		self.parse_last_excludes()
		self.parse_rbdir_exclude()

	def parse_catch_error(self, exc):
		"""Deal with selection error exc"""
		if isinstance(exc, FilePrefixError):
			log.Log.FatalError(
"""Fatal Error: The file specification
    '%s'
cannot match any files in the base directory
    '%s'
Useful file specifications begin with the base directory or some
pattern (such as '**') which matches the base directory.""" %
			(exc, self.prefix))
		elif isinstance(e, GlobbingError):
			log.Log.FatalError("Fatal Error while processing expression\n"
							   "%s" % exc)
		else: raise

	def parse_rbdir_exclude(self):
		"""Add exclusion of rdiff-backup-data dir to front of list"""
		self.add_selection_func(
			self.glob_get_tuple_sf(("rdiff-backup-data",), 0), 1)

	def parse_last_excludes(self):
		"""Exit with error if last selection function isn't an exclude"""
		if (self.selection_functions and
			not self.selection_functions[-1].exclude):
			log.Log.FatalError(
"""Last selection expression:
    %s
only specifies that files be included.  Because the default is to
include all files, the expression is redundant.  Exiting because this
probably isn't what you meant.""" %
			(self.selection_functions[-1].name,))

	def add_selection_func(self, sel_func, add_to_start = None):
		"""Add another selection function at the end or beginning"""
		if add_to_start: self.selection_functions.insert(0, sel_func)
		else: self.selection_functions.append(sel_func)

	def filelist_get_sf(self, filelist_fp, inc_default, filelist_name):
		"""Return selection function by reading list of files

		The format of the filelist is documented in the man page.
		filelist_fp should be an (open) file object.
		inc_default should be true if this is an include list,
		false for an exclude list.
		filelist_name is just a string used for logging.

		"""
		log.Log("Reading filelist %s" % filelist_name, 4)
		tuple_list, something_excluded = \
					self.filelist_read(filelist_fp, inc_default, filelist_name)
		log.Log("Sorting filelist %s" % filelist_name, 4)
		tuple_list.sort()
		i = [0] # We have to put index in list because of stupid scoping rules

		def selection_function(rp):
			while 1:
				if i[0] >= len(tuple_list): return None
				include, move_on = \
						 self.filelist_pair_match(rp, tuple_list[i[0]])
				if move_on:
					i[0] += 1
					if include is None: continue # later line may match
				return include

		selection_function.exclude = something_excluded or inc_default == 0
		selection_function.name = "Filelist: " + filelist_name
		return selection_function

	def filelist_read(self, filelist_fp, include, filelist_name):
		"""Read filelist from fp, return (tuplelist, something_excluded)"""
		prefix_warnings = [0]
		def incr_warnings(exc):
			"""Warn if prefix is incorrect"""
			prefix_warnings[0] += 1
			if prefix_warnings[0] < 6:
				log.Log("Warning: file specification '%s' in filelist %s\n"
						"doesn't start with correct prefix %s.  Ignoring." %
						(exc, filelist_name, self.prefix), 2)
				if prefix_warnings[0] == 5:
					log.Log("Future prefix errors will not be logged.", 2)

		something_excluded, tuple_list = None, []
		separator = Globals.null_separator and "\0" or "\n"
		for line in filelist_fp.read().split(separator):
			if not line: continue # skip blanks
			try: tuple = self.filelist_parse_line(line, include)
			except FilePrefixError, exc:
				incr_warnings(exc)
				continue
			tuple_list.append(tuple)
			if not tuple[1]: something_excluded = 1
		if filelist_fp.close():
			log.Log("Error closing filelist %s" % filelist_name, 2)
		return (tuple_list, something_excluded)

	def filelist_parse_line(self, line, include):
		"""Parse a single line of a filelist, returning a pair

		pair will be of form (index, include), where index is another
		tuple, and include is 1 if the line specifies that we are
		including a file.  The default is given as an argument.
		prefix is the string that the index is relative to.

		"""
		if line[:2] == "+ ": # Check for "+ "/"- " syntax
			include = 1
			line = line[2:]
		elif line[:2] == "- ":
			include = 0
			line = line[2:]

		if not line.startswith(self.prefix): raise FilePrefixError(line)
		line = line[len(self.prefix):] # Discard prefix
		index = tuple(filter(lambda x: x, line.split("/"))) # remove empties
		return (index, include)

	def filelist_pair_match(self, rp, pair):
		"""Matches a filelist tuple against a rpath

		Returns a pair (include, move_on).  include is None if the
		tuple doesn't match either way, and 0/1 if the tuple excludes
		or includes the rpath.

		move_on is true if the tuple cannot match a later index, and
		so we should move on to the next tuple in the index.

		"""
		index, include = pair
		if include == 1:
			if index < rp.index: return (None, 1)
			if index == rp.index: return (1, 1)
			elif index[:len(rp.index)] == rp.index:
				return (1, None) # /foo/bar implicitly includes /foo
			else: return (None, None) # rp greater, not initial sequence
		elif include == 0:
			if rp.index[:len(index)] == index:
				return (0, None) # /foo implicitly excludes /foo/bar
			elif index < rp.index: return (None, 1)
			else: return (None, None) # rp greater, not initial sequence
		else: assert 0, "Include is %s, should be 0 or 1" % (include,)

	def filelist_globbing_get_sfs(self, filelist_fp, inc_default, list_name):
		"""Return list of selection functions by reading fileobj

		filelist_fp should be an open file object
		inc_default is true iff this is an include list
		list_name is just the name of the list, used for logging
		See the man page on --[include/exclude]-globbing-filelist

		"""
		log.Log("Reading globbing filelist %s" % list_name, 4)
		separator = Globals.null_separator and "\0" or "\n"
		for line in filelist_fp.read().split(separator):
			if not line: continue # skip blanks
			if line[:2] == "+ ": yield self.glob_get_sf(line[2:], 1)
			elif line[:2] == "- ": yield self.glob_get_sf(line[2:], 0)
			else: yield self.glob_get_sf(line, inc_default)

	def other_filesystems_get_sf(self, include):
		"""Return selection function matching files on other filesystems"""
		assert include == 0 or include == 1
		root_devloc = self.rpath.getdevloc()
		def sel_func(rp):
			if rp.getdevloc() == root_devloc: return None
			else: return include
		sel_func.exclude = not include
		sel_func.name = "Match other filesystems"
		return sel_func

	def regexp_get_sf(self, regexp_string, include):
		"""Return selection function given by regexp_string"""
		assert include == 0 or include == 1
		try: regexp = re.compile(regexp_string)
		except:
			log.Log("Error compiling regular expression %s" % regexp_string, 1)
			raise
		
		def sel_func(rp):
			if regexp.search(rp.path): return include
			else: return None

		sel_func.exclude = not include
		sel_func.name = "Regular expression: %s" % regexp_string
		return sel_func

	def presence_get_sf(self, presence_filename, include):
		"""Return selection function given by a file if present"""
		assert include == 0 or include == 1

		def sel_func(rp):
			if rp.isdir() and rp.readable() and \
					rp.append(presence_filename).lstat():
				return include
			return None

		sel_func.exclude = not include
		sel_func.name = "Presence file: %s" % presence_filename
		return sel_func

	def gen_get_sf(self, pred, include, name):
		"""Returns a selection function that uses pred to test
		
		RPath is matched if pred returns true on it.  Name is a string
		summarizing the test to the user.

		"""
		def sel_func(rp):
			if pred(rp): return include
			return None
		sel_func.exclude = not include
		sel_func.name = (include and "include " or "exclude ") + name
		return sel_func

	def devfiles_get_sf(self, include):
		"""Return a selection function matching all dev files"""
		return self.gen_get_sf(rpath.RORPath.isdev, include, "device files")

 	def symlinks_get_sf(self, include):
 		"""Return a selection function matching all symlinks"""
		return self.gen_get_sf(rpath.RORPath.issym, include, "symbolic links")

 	def sockets_get_sf(self, include):
 		"""Return a selection function matching all sockets"""
		return self.gen_get_sf(rpath.RORPath.issock, include, "socket files")

 	def fifos_get_sf(self, include):
 		"""Return a selection function matching all fifos"""
		return self.gen_get_sf(rpath.RORPath.isfifo, include, "fifo files")

	def special_get_sf(self, include):
		"""Return sel function matching sockets, symlinks, sockets, devs"""
		def sel_func(rp):
			if rp.issym() or rp.issock() or rp.isfifo() or rp.isdev():
				return include
			else: return None
		sel_func.exclude = not include
		sel_func.name = (include and "include" or "exclude") + " special files"
		return sel_func

	def size_get_sf(self, min_max, sizestr):
		"""Return selection function given by filesize"""
		size = int(sizestr)
		assert size > 0
		if min_max: sel_func = lambda rp: (rp.getsize() <= size)
		else: sel_func = lambda rp: (rp.getsize() >= size)
		sel_func.exclude = 1
		sel_func.name = "%s size %d" % (min_max and "Maximum" or "Minimum", size)
		return sel_func

	def glob_get_sf(self, glob_str, include):
		"""Return selection function given by glob string"""
		assert include == 0 or include == 1
		if glob_str == "**": sel_func = lambda rp: include
		elif not self.glob_re.match(glob_str): # normal file
			sel_func = self.glob_get_filename_sf(glob_str, include)
		else: sel_func = self.glob_get_normal_sf(glob_str, include)

		sel_func.exclude = not include
		sel_func.name = "Command-line %s glob: %s" % \
						(include and "include" or "exclude", glob_str)
		return sel_func

	def glob_get_filename_sf(self, filename, include):
		"""Get a selection function given a normal filename

		Some of the parsing is better explained in
		filelist_parse_line.  The reason this is split from normal
		globbing is things are a lot less complicated if no special
		globbing characters are used.

		"""
		if not filename.startswith(self.prefix):
			raise FilePrefixError(filename)
		index = tuple(filter(lambda x: x,
							 filename[len(self.prefix):].split("/")))
		return self.glob_get_tuple_sf(index, include)

	def glob_get_tuple_sf(self, tuple, include):
		"""Return selection function based on tuple"""
		def include_sel_func(rp):
			if (rp.index == tuple[:len(rp.index)] or
				rp.index[:len(tuple)] == tuple):
				return 1 # /foo/bar implicitly matches /foo, vice-versa
			else: return None

		def exclude_sel_func(rp):
			if rp.index[:len(tuple)] == tuple:
				return 0 # /foo excludes /foo/bar, not vice-versa
			else: return None

		if include == 1: sel_func = include_sel_func
		elif include == 0: sel_func = exclude_sel_func
		sel_func.exclude = not include
		sel_func.name = "Tuple select %s" % (tuple,)
		return sel_func

	def glob_get_normal_sf(self, glob_str, include):
		"""Return selection function based on glob_str

		The basic idea is to turn glob_str into a regular expression,
		and just use the normal regular expression.  There is a
		complication because the selection function should return '2'
		(scan) for directories which may contain a file which matches
		the glob_str.  So we break up the glob string into parts, and
		any file which matches an initial sequence of glob parts gets
		scanned.

		Thanks to Donovan Baarda who provided some code which did some
		things similar to this.

		"""
		if glob_str.lower().startswith("ignorecase:"):
			re_comp = lambda r: re.compile(r, re.I | re.S)
			glob_str = glob_str[len("ignorecase:"):]
		else: re_comp = lambda r: re.compile(r, re.S)

		# matches what glob matches and any files in directory
		glob_comp_re = re_comp("^%s($|/)" % self.glob_to_re(glob_str))

		if glob_str.find("**") != -1:
			glob_str = glob_str[:glob_str.find("**")+2] # truncate after **

		scan_comp_re = re_comp("^(%s)$" %
							   "|".join(self.glob_get_prefix_res(glob_str)))

		def include_sel_func(rp):
			if glob_comp_re.match(rp.path): return 1
			elif scan_comp_re.match(rp.path): return 2
			else: return None

		def exclude_sel_func(rp):
			if glob_comp_re.match(rp.path): return 0
			else: return None

		# Check to make sure prefix is ok
		if not include_sel_func(self.rpath): raise FilePrefixError(glob_str)
		
		if include: return include_sel_func
		else: return exclude_sel_func

	def glob_get_prefix_res(self, glob_str):
		"""Return list of regexps equivalent to prefixes of glob_str"""
		glob_parts = glob_str.split("/")
		if "" in glob_parts[1:-1]: # "" OK if comes first or last, as in /foo/
			raise GlobbingError("Consecutive '/'s found in globbing string "
								+ glob_str)

		prefixes = map(lambda i: "/".join(glob_parts[:i+1]),
					   range(len(glob_parts)))
		# we must make exception for root "/", only dir to end in slash
		if prefixes[0] == "": prefixes[0] = "/"
		return map(self.glob_to_re, prefixes)

	def glob_to_re(self, pat):
		"""Returned regular expression equivalent to shell glob pat

		Currently only the ?, *, [], and ** expressions are supported.
		Ranges like [a-z] are also currently unsupported.  These special
		characters can be quoted by prepending them with a backslash.

		This function taken with minor modifications from efnmatch.py
		by Donovan Baarda.

		"""
		i, n, res = 0, len(pat), ''
		while i < n:
			c, s = pat[i], pat[i:i+2]
			i = i+1
			if c == '\\':
				res = res + re.escape(s[-1])
				i = i + 1
			elif s == '**':
				res = res + '.*'
				i = i + 1
			elif c == '*': res = res + '[^/]*'
			elif c == '?': res = res + '[^/]'
			elif c == '[':
				j = i
				if j < n and pat[j] in '!^': j = j+1
				if j < n and pat[j] == ']': j = j+1
				while j < n and pat[j] != ']': j = j+1
				if j >= n: res = res + '\\[' # interpret the [ literally
				else: # Deal with inside of [..]
					stuff = pat[i:j].replace('\\','\\\\')
					i = j+1
					if stuff[0] in '!^': stuff = '^' + stuff[1:]
					res = res + '[' + stuff + ']'
			else: res = res + re.escape(c)
		return res


class FilterIter:
	"""Filter rorp_iter using a Select object, removing excluded rorps"""
	def __init__(self, select, rorp_iter):
		"""Constructor

		Input is the Select object to use and the iter of rorps to be
		filtered.  The rorps will be converted to rps using the Select
		base.

		"""
		self.rorp_iter = rorp_iter
		self.base_rp = select.rpath
		self.stored_rorps = []
		self.ITR = rorpiter.IterTreeReducer(FilterIterITRB,
											[select.Select, self.stored_rorps])
		self.itr_finished = 0

	def __iter__(self): return self

	def next(self):
		"""Return next object, or StopIteration"""
		while not self.stored_rorps:
			try: next_rorp = self.rorp_iter.next()
			except StopIteration:
				if self.itr_finished: raise
				else:
					self.ITR.Finish()
					self.itr_finished = 1
			else:
				next_rp = rpath.RPath(self.base_rp.conn, self.base_rp.base,
									  next_rorp.index, next_rorp.data)
				self.ITR(next_rorp.index, next_rp, next_rorp)
		return self.stored_rorps.pop(0)

class FilterIterITRB(rorpiter.ITRBranch):
	"""ITRBranch used in above FilterIter class

	The reason this is necessary is because for directories sometimes
	we don't know whether a rorp is excluded until we see what is in
	the directory.

	"""
	def __init__(self, select, rorp_cache):
		"""Initialize FilterIterITRB.  Called by IterTreeReducer.

		select should be the relevant Select object used to test the
		rps.  rorp_cache is the list rps should be appended to if they
		aren't excluded.

		"""
		self.select, self.rorp_cache = select, rorp_cache
		self.branch_excluded = None
		self.base_queue = None # holds branch base while examining contents

	def can_fast_process(self, index, next_rp, next_rorp):
		return not next_rp.isdir()

	def fast_process(self, index, next_rp, next_rorp):
		"""For ordinary files, just append if select is positive"""
		if self.branch_excluded: return
		s = self.select(next_rp)
		if s == 1:
			if self.base_queue:
				self.rorp_cache.append(self.base_queue)
				self.base_queue = None
			self.rorp_cache.append(next_rorp)
		else: assert s == 0, "Unexpected select value %s" % (s,)

	def start_process(self, index, next_rp, next_rorp):
		s = self.select(next_rp)
		if s == 0: self.branch_excluded = 1
		elif s == 1: self.rorp_cache.append(next_rorp)
		else:
			assert s == 2, s
			self.base_queue = next_rorp




# Copyright 2008 Fred Gansevles <fred@betterbe.com>
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

from __future__ import generators
import C, metadata, re, rorpiter, rpath, log

try:
	from win32security import *
	import pywintypes
except ImportError:
	GROUP_SECURITY_INFORMATION = 0
	OWNER_SECURITY_INFORMATION = 0
	DACL_SECURITY_INFORMATION = 0

class ACL:
	flags = (GROUP_SECURITY_INFORMATION|
		 OWNER_SECURITY_INFORMATION|
		 DACL_SECURITY_INFORMATION)

	def __init__(self, index=()):
		self.__acl = ""
		self.index = index

	def get_indexpath(self): return self.index and '/'.join(self.index) or '.'

	def load_from_rp(self, rp, skip_inherit_only = True):
		self.index = rp.index
		try:
			sd = rp.conn.win32security. \
					GetNamedSecurityInfo(rp.path, SE_FILE_OBJECT, ACL.flags)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to read ACL from %s: %s"
					% (repr(rp.path), exc), 4)
			return

		if skip_inherit_only:
			# skip the inherit_only aces
			acl = sd.GetSecurityDescriptorDacl()
			if acl:
				n = acl.GetAceCount()
				# traverse the ACL in reverse, so the indices stay correct
				while n:
					n -= 1
					ace_flags = acl.GetAce(n)[0][1]
					if ace_flags & INHERIT_ONLY_ACE:
						acl.DeleteAce(n)
			sd.SetSecurityDescriptorDacl(1, acl, 0)

			if ACL.flags & SACL_SECURITY_INFORMATION:
				acl = sd.GetSecurityDescriptorSacl()
				if acl:
					n = acl.GetAceCount()
					# traverse the ACL in reverse, so the indices stay correct
					while n:
						n -= 1
						ace_flags = acl.GetAce(n)[0][1]
						if ace_flags & INHERIT_ONLY_ACE:
							acl.DeleteAce(n)
					sd.SetSecurityDescriptorSacl(1, acl, 0)

		if not sd.GetSecurityDescriptorDacl():
				sd.SetSecurityDescriptorDacl(0, None, 0)
		if (ACL.flags & SACL_SECURITY_INFORMATION) and not \
				sd.GetSecurityDescriptorSacl():
			sd.SetSecurityDescriptorSacl(0, None, 0)

		try:
			self.__acl = \
				rp.conn.win32security. \
					ConvertSecurityDescriptorToStringSecurityDescriptor(sd,
					SDDL_REVISION_1, ACL.flags)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to convert ACL from %s to string: %s"
					% (repr(rp.path), exc), 4)
			self.__acl = ''

	def clear_rp(self, rp):
		# not sure how to interpret this
		# I'll just clear all acl-s from rp.path
		try:
			sd = rp.conn.win32security. \
					GetNamedSecurityInfo(rp.path, SE_FILE_OBJECT, ACL.flags)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to read ACL from %s for clearing: %s"
					% (repr(rp.path), exc), 4)

		acl = sd.GetSecurityDescriptorDacl()
		if acl:
			n = acl.GetAceCount()
			# traverse the ACL in reverse, so the indices stay correct
			while n:
				n -= 1
				acl.DeleteAce(n)
			sd.SetSecurityDescriptorDacl(0, acl, 0)

		if ACL.flags & SACL_SECURITY_INFORMATION:
			acl = sd.GetSecurityDescriptorSacl()
			if acl:
				n = acl.GetAceCount()
				# traverse the ACL in reverse, so the indices stay correct
				while n:
					n -= 1
					acl.DeleteAce(n)
				sd.SetSecurityDescriptorSacl(0, acl, 0)

		try:
			rp.conn.win32security. \
				SetNamedSecurityInfo(rp.path, SE_FILE_OBJECT, ACL.flags,
				sd.GetSecurityDescriptorOwner(),sd.GetSecurityDescriptorGroup(),
				sd.GetSecurityDescriptorDacl(),
				(ACL.flags & SACL_SECURITY_INFORMATION) and 
					sd.GetSecurityDescriptorSacl() or None)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to set ACL on %s after clearing: %s"
					% (repr(rp.path), exc), 4)

	def write_to_rp(self, rp):
		if not self.__acl:
			return

		try:
			sd = rp.conn.win32security. \
				ConvertStringSecurityDescriptorToSecurityDescriptor(
					self.__acl, SDDL_REVISION_1)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to convert string %s to ACL: %s"
				% (repr(self.__acl), exc), 4)

		# Enable next block of code for dirs after we have a mechanism in
		# backup.py (and similar) to do a first pass to see if a directory
		# has SE_DACL_PROTECTED. In that case, we will need to
		#		1) dest_rorp.write_win_acl(source_rorp.get_win_acl())
		#				--> And clear existing dest_rorp one while doing so
		#		2) Check if backup user has Admin privs to write dest_rorp
		#				--> May need to use Win32 AccessCheck() API
		#		3) If not, add Admin write privs to dest_rorp and add dir
		#				to dir_perms_list-equivalent
		#		4) THEN, allow the pre_process() function to finish and the
		#				files be copied over. Those files which wish to
		#				will now inherit the correct ACE objects.
		#		5) If dir was on dir_perms_list-equivalent, drop the write
		#				write permission we added.
		#		6) When copy_attribs is called in end_process, make sure
		#				that the write_win_acl() call isn't made this time
		# The reason we will need to do this is because otherwise, the files
		# which are created during step 4 will reference the ACE entries
		# which we clear during step 6. We need to clear them *before* the
		# children files/subdirs are created and generate the appropriate
		# DACL so the inheritance magic can happen during step 4.

		(flags, revision) = sd.GetSecurityDescriptorControl()
		if (not rp.isdir() and flags & SE_DACL_PROTECTED):
			self.clear_rp(rp)

		try:
			rp.conn.win32security. \
				SetNamedSecurityInfo(rp.path, SE_FILE_OBJECT, ACL.flags,
				sd.GetSecurityDescriptorOwner(),sd.GetSecurityDescriptorGroup(),
				sd.GetSecurityDescriptorDacl(),
				(ACL.flags & SACL_SECURITY_INFORMATION) and 
					sd.GetSecurityDescriptorSacl() or None)
		except (OSError, IOError, pywintypes.error), exc:
			log.Log("Warning: unable to set ACL on %s: %s"
					% (repr(rp.path), exc), 4)

	def __str__(self):
		return '# file: %s\n%s\n' % \
				(C.acl_quote(self.get_indexpath()), unicode(self.__acl))

	def from_string(self, acl_str):
		lines = acl_str.splitlines()
		if len(lines) != 2 or not lines[0][:8] == "# file: ":
			raise metadata.ParsingError("Bad record beginning: " + lines[0][:8])
		filename = lines[0][8:]
		if filename == '.': self.index = ()
		else: self.index = tuple(C.acl_unquote(filename).split('/'))
		self.__acl = lines[1]

def Record2WACL(record):
	acl = ACL()
	acl.from_string(record)
	return acl

def WACL2Record(wacl):
	return unicode(wacl)

class WACLExtractor(metadata.FlatExtractor):
	"""Iterate ExtendedAttributes objects from the WACL information file"""
	record_boundary_regexp = re.compile('(?:\\n|^)(# file: (.*?))\\n')
	record_to_object = staticmethod(Record2WACL)
	def filename_to_index(self, filename):
		"""Convert possibly quoted filename to index tuple"""
		if filename == '.': return ()
		else: return tuple(C.acl_unquote(filename).split('/'))

class WinAccessControlListFile(metadata.FlatFile):
	"""Store/retrieve ACLs from extended_attributes file"""
	_prefix = "win_access_control_lists"
	_extractor = WACLExtractor
	_object_to_record = staticmethod(WACL2Record)

def join_wacl_iter(rorp_iter, wacl_iter):
	"""Update a rorp iter by adding the information from acl_iter"""
	for rorp, wacl in rorpiter.CollateIterators(rorp_iter, wacl_iter):
		assert rorp, "Missing rorp for index %s" % (wacl.index,)
		if not wacl: wacl = ACL(rorp.index)
		rorp.set_win_acl(unicode(wacl))
		yield rorp
	
def rpath_acl_win_get(rpath):
	acl = ACL()
	acl.load_from_rp(rpath)
	return unicode(acl)
rpath.win_acl_get = rpath_acl_win_get

def rpath_get_blank_win_acl(index):
	acl = ACL(index)
	return unicode(acl)
rpath.get_blank_win_acl = rpath_get_blank_win_acl

def rpath_set_win_acl(rp, acl_str):
	acl = ACL()
	acl.from_string(acl_str)
	acl.write_to_rp(rp)
rpath.write_win_acl = rpath_set_win_acl

def init_acls():
	# A process that tries to read or write a SACL needs
	# to have and enable the SE_SECURITY_NAME privilege.
	# And inorder to backup/restore, the SE_BACKUP_NAME and
	# SE_RESTORE_NAME privileges are needed.
	import win32api
	try:
		hnd = OpenProcessToken(win32api.GetCurrentProcess(),
			TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY)
	except win32api.error, exc:
		log.Log("Warning: unable to open Windows process token: %s"
				% exc, 5)
		return
	try:
		try:
			lpv = lambda priv: LookupPrivilegeValue(None, priv)
			# enable the SE_*_NAME privileges 
			SecurityName = lpv(SE_SECURITY_NAME)
			AdjustTokenPrivileges(hnd, False, [
				(SecurityName, SE_PRIVILEGE_ENABLED),
				(lpv(SE_BACKUP_NAME), SE_PRIVILEGE_ENABLED),
				(lpv(SE_RESTORE_NAME), SE_PRIVILEGE_ENABLED)
				])
		except win32api.error, exc:
			log.Log("Warning: unable to enable SE_*_NAME privileges: %s"
				% exc, 5)
			return
		for name, enabled in GetTokenInformation(hnd, TokenPrivileges):
			if name == SecurityName and enabled:
				# now we *may* access the SACL (sigh)
				ACL.flags |= SACL_SECURITY_INFORMATION
				break
	finally:
		win32api.CloseHandle(hnd)



# Copyright 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Contains a file wrapper that returns a hash on close"""

# Until rdiff-backup is ported to Python 3 (or abandons support for versions
# below Python 2.5), we'll ignore the warning about the deprecated sha module
import warnings
warnings.filterwarnings("ignore", ".*sha module.*", DeprecationWarning)

import sha
import Globals

class FileWrapper:
	"""Wrapper around a file-like object

	Only use this with files that will be read through in a single
	pass and then closed.  (There is no seek().)  When you close it,
	return value will be a Report.

	Currently this just calculates a sha1sum of the datastream.

	"""
	def __init__(self, fileobj):
		self.fileobj = fileobj
		self.sha1 = sha.new()
		self.closed = 0

	def read(self, length = -1):
		assert not self.closed
		buf = self.fileobj.read(length)
		self.sha1.update(buf)
		return buf

	def close(self):
		return Report(self.fileobj.close(), self.sha1.hexdigest())


class Report:
	"""Hold final information about a byte stream"""
	def __init__(self, close_val, sha1_digest):
		assert not close_val # For now just assume inner file closes correctly
		self.sha1_digest = sha1_digest


def compute_sha1(rp, compressed = 0):
	"""Return the hex sha1 hash of given rpath"""
	assert rp.conn is Globals.local_connection # inefficient not to do locally
	digest = compute_sha1_fp(rp.open("rb", compressed))
	rp.set_sha1(digest)
	return digest

def compute_sha1_fp(fp, compressed = 0):
	"""Return hex sha1 hash of given file-like object"""
	blocksize = Globals.blocksize
	fw = FileWrapper(fp)
	while 1:
		if not fw.read(blocksize): break
	return fw.close().sha1_digest





# Copyright 2002, 2003, 2004, 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Read increment files and restore to original"""

from __future__ import generators
import tempfile, os, cStringIO
import static, rorpiter, FilenameMapping

class RestoreError(Exception): pass

def Restore(mirror_rp, inc_rpath, target, restore_to_time):
	"""Recursively restore mirror and inc_rpath to target at rest_time"""
	MirrorS = mirror_rp.conn.restore.MirrorStruct
	TargetS = target.conn.restore.TargetStruct

	MirrorS.set_mirror_and_rest_times(restore_to_time)
	MirrorS.initialize_rf_cache(mirror_rp, inc_rpath)
	target_iter = TargetS.get_initial_iter(target)
	diff_iter = MirrorS.get_diffs(target_iter)
	TargetS.patch(target, diff_iter)
	MirrorS.close_rf_cache()

def get_inclist(inc_rpath):
	"""Returns increments with given base"""
	dirname, basename = inc_rpath.dirsplit()
	if Globals.chars_to_quote: basename = FilenameMapping.unquote(basename)
	parent_dir = inc_rpath.__class__(inc_rpath.conn, dirname, ())
	if not parent_dir.isdir(): return [] # inc directory not created yet
	index = inc_rpath.index

	inc_list = []
	for filename in parent_dir.listdir():
		inc_info = rpath.get_incfile_info(filename)
		if inc_info and inc_info[3] == basename:
			inc = parent_dir.append(filename)
			assert inc.isincfile()
			inc_list.append(inc)
	return inc_list

def ListChangedSince(mirror_rp, inc_rp, restore_to_time):
	"""List the changed files under mirror_rp since rest time

	Notice the output is an iterator of RORPs.  We do this because we
	want to give the remote connection the data in buffered
	increments, and this is done automatically for rorp iterators.
	Encode the lines in the first element of the rorp's index.

	"""
	assert mirror_rp.conn is Globals.local_connection, "Run locally only"
	MirrorStruct.set_mirror_and_rest_times(restore_to_time)
	MirrorStruct.initialize_rf_cache(mirror_rp, inc_rp)

	old_iter = MirrorStruct.get_mirror_rorp_iter(MirrorStruct._rest_time, 1)
	cur_iter = MirrorStruct.get_mirror_rorp_iter(MirrorStruct._mirror_time, 1)
	collated = rorpiter.Collate2Iters(old_iter, cur_iter)
	for old_rorp, cur_rorp in collated:
		if not old_rorp: change = "new"
		elif not cur_rorp: change = "deleted"
		elif old_rorp == cur_rorp: continue
		else: change = "changed"
		path_desc = (old_rorp and old_rorp.get_indexpath() or
					 cur_rorp.get_indexpath())
		yield rpath.RORPath(("%-7s %s" % (change, path_desc),))
	MirrorStruct.close_rf_cache()

def ListAtTime(mirror_rp, inc_rp, time):
	"""List the files in archive at the given time

	Output is a RORP Iterator with info in index.  See ListChangedSince.

	"""
	assert mirror_rp.conn is Globals.local_connection, "Run locally only"
	MirrorStruct.set_mirror_and_rest_times(time)
	MirrorStruct.initialize_rf_cache(mirror_rp, inc_rp)
	old_iter = MirrorStruct.get_mirror_rorp_iter()
	for rorp in old_iter: yield rorp
	

class MirrorStruct:
	"""Hold functions to be run on the mirror side"""
	# If selection command line arguments given, use Select here
	_select = None
	# This will be set to the time of the current mirror
	_mirror_time = None
	# This will be set to the exact time to restore to (not restore_to_time)
	_rest_time = None
	
	def set_mirror_and_rest_times(cls, restore_to_time):
		"""Set class variabels _mirror_time and _rest_time on mirror conn"""
		MirrorStruct._mirror_time = cls.get_mirror_time()
		MirrorStruct._rest_time = cls.get_rest_time(restore_to_time)

	def get_mirror_time(cls):
		"""Return time (in seconds) of latest mirror"""
		cur_mirror_incs = get_inclist(Globals.rbdir.append("current_mirror"))
		if not cur_mirror_incs:
			log.Log.FatalError("Could not get time of current mirror")
		elif len(cur_mirror_incs) > 1:
			log.Log("Warning, two different times for current mirror found", 2)
		return cur_mirror_incs[0].getinctime()

	def get_rest_time(cls, restore_to_time):
		"""Return older time, if restore_to_time is in between two inc times

		There is a slightly tricky reason for doing this: The rest of the
		code just ignores increments that are older than restore_to_time.
		But sometimes we want to consider the very next increment older
		than rest time, because rest_time will be between two increments,
		and what was actually on the mirror side will correspond to the
		older one.

		So if restore_to_time is inbetween two increments, return the
		older one.

		"""
		inctimes = cls.get_increment_times()
		older_times = filter(lambda time: time <= restore_to_time, inctimes)
		if older_times: return max(older_times)
		else: # restore time older than oldest increment, just return that
			return min(inctimes)

	def get_increment_times(cls, rp = None):
		"""Return list of times of backups, including current mirror

		Take the total list of times from the increments.<time>.dir
		file and the mirror_metadata file.  Sorted ascending.

		"""
		# use dictionary to remove dups
		if not cls._mirror_time: d = {cls.get_mirror_time(): None}
		else: d = {cls._mirror_time: None}
		if not rp or not rp.index: rp = Globals.rbdir.append("increments")
		for inc in get_inclist(rp): d[inc.getinctime()] = None
		for inc in get_inclist(Globals.rbdir.append("mirror_metadata")):
			d[inc.getinctime()] = None
		return_list = d.keys()
		return_list.sort()
		return return_list

	def initialize_rf_cache(cls, mirror_base, inc_base):
		"""Set cls.rf_cache to CachedRF object"""
		inc_list = get_inclist(inc_base)
		rf = RestoreFile(mirror_base, inc_base, get_inclist(inc_base))
		cls.mirror_base, cls.inc_base = mirror_base, inc_base
		cls.root_rf = rf
		cls.rf_cache = CachedRF(rf)

	def close_rf_cache(cls):
		"""Run anything remaining on CachedRF object"""
		cls.rf_cache.close()

	def get_mirror_rorp_iter(cls, rest_time = None, require_metadata = None):
		"""Return iter of mirror rps at given restore time

		Usually we can use the metadata file, but if this is
		unavailable, we may have to build it from scratch.

		If the cls._select object is set, use it to filter out the
		unwanted files from the metadata_iter.

		"""
		if rest_time is None: rest_time = cls._rest_time

		metadata.SetManager()
		rorp_iter = metadata.ManagerObj.GetAtTime(rest_time,
												  cls.mirror_base.index)
		if not rorp_iter:
			if require_metadata:
				log.Log.FatalError("Mirror metadata not found")
			log.Log("Warning: Mirror metadata not found, "
					"reading from directory", 2)
			rorp_iter = cls.get_rorp_iter_from_rf(cls.root_rf)

		if cls._select:
			rorp_iter = selection.FilterIter(cls._select, rorp_iter)
		return rorp_iter

	def set_mirror_select(cls, target_rp, select_opts, *filelists):
		"""Initialize the mirror selection object"""
		assert select_opts, "If no selection options, don't use selector"
		cls._select = selection.Select(target_rp)
		cls._select.ParseArgs(select_opts, filelists)

	def get_rorp_iter_from_rf(cls, rf):
		"""Recursively yield mirror rorps from rf"""
		rorp = rf.get_attribs()
		yield rorp
		if rorp.isdir():
			for sub_rf in rf.yield_sub_rfs():
				for attribs in cls.get_rorp_iter_from_rf(sub_rf):
					yield attribs

	def subtract_indicies(cls, index, rorp_iter):
		"""Subtract index from index of each rorp in rorp_iter

		subtract_indicies and add_indicies are necessary because we
		may not be restoring from the root index.

		"""
		if index == (): return rorp_iter
		def get_iter():
			for rorp in rorp_iter:
				assert rorp.index[:len(index)] == index, (rorp.index, index)
				rorp.index = rorp.index[len(index):]
				yield rorp
		return get_iter()

	def get_diffs(cls, target_iter):
		"""Given rorp iter of target files, return diffs

		Here the target_iter doesn't contain any actual data, just
		attribute listings.  Thus any diffs we generate will be
		snapshots.

		"""
		mir_iter = cls.subtract_indicies(cls.mirror_base.index,
										 cls.get_mirror_rorp_iter())
		collated = rorpiter.Collate2Iters(mir_iter, target_iter)
		return cls.get_diffs_from_collated(collated)

	def get_diffs_from_collated(cls, collated):
		"""Get diff iterator from collated"""
		for mir_rorp, target_rorp in collated:
			if Globals.preserve_hardlinks and mir_rorp:
				Hardlink.add_rorp(mir_rorp, target_rorp)
			if (not target_rorp or not mir_rorp or
				not mir_rorp == target_rorp or
				(Globals.preserve_hardlinks and not
				 Hardlink.rorp_eq(mir_rorp, target_rorp))):
				diff = cls.get_diff(mir_rorp, target_rorp)
			else: diff = None
			if Globals.preserve_hardlinks and mir_rorp:
				Hardlink.del_rorp(mir_rorp)
			if diff: yield diff

	def get_diff(cls, mir_rorp, target_rorp):
		"""Get a diff for mir_rorp at time"""
		if not mir_rorp: mir_rorp = rpath.RORPath(target_rorp.index)
		elif Globals.preserve_hardlinks and Hardlink.islinked(mir_rorp):
			mir_rorp.flaglinked(Hardlink.get_link_index(mir_rorp))
		elif mir_rorp.isreg():
			expanded_index = cls.mirror_base.index + mir_rorp.index
			file_fp = cls.rf_cache.get_fp(expanded_index, mir_rorp)
			mir_rorp.setfile(hash.FileWrapper(file_fp))
		mir_rorp.set_attached_filetype('snapshot')
		return mir_rorp

static.MakeClass(MirrorStruct)


class TargetStruct:
	"""Hold functions to be run on the target side when restoring"""
	_select = None
	def set_target_select(cls, target, select_opts, *filelists):
		"""Return a selection object iterating the rorpaths in target"""
		cls._select = selection.Select(target)
		cls._select.ParseArgs(select_opts, filelists)
		cls._select.set_iter()

	def get_initial_iter(cls, target):
		"""Return selector previously set with set_initial_iter"""
		return cls._select or selection.Select(target).set_iter()

	def patch(cls, target, diff_iter):
		"""Patch target with the diffs from the mirror side

		This function and the associated ITRB is similar to the
		patching code in backup.py, but they have different error
		correction requirements, so it seemed easier to just repeat it
		all in this module.

		"""
		ITR = rorpiter.IterTreeReducer(PatchITRB, [target])
		for diff in rorpiter.FillInIter(diff_iter, target):
			log.Log("Processing changed file " + diff.get_indexpath(), 5)
			ITR(diff.index, diff)
		ITR.Finish()
		target.setdata()

static.MakeClass(TargetStruct)


class CachedRF:
	"""Store RestoreFile objects until they are needed

	The code above would like to pretend it has random access to RFs,
	making one for a particular index at will.  However, in general
	this involves listing and filtering a directory, which can get
	expensive.

	Thus, when a CachedRF retrieves an RestoreFile, it creates all the
	RFs of that directory at the same time, and doesn't have to
	recalculate.  It assumes the indicies will be in order, so the
	cache is deleted if a later index is requested.

	"""
	def __init__(self, root_rf):
		"""Initialize CachedRF, self.rf_list variable"""
		self.root_rf = root_rf
		self.rf_list = [] # list should filled in index order
		if Globals.process_uid != 0:
			self.perm_changer = PermissionChanger(root_rf.mirror_rp)

	def list_rfs_in_cache(self, index):
		"""Used for debugging, return indicies of cache rfs for printing"""
		s1 = "-------- Cached RF for %s -------" % (index,)
		s2 = " ".join([str(rf.index) for rf in self.rf_list])
		s3 = "--------------------------"
		return "\n".join((s1, s2, s3))

	def get_rf(self, index, mir_rorp = None):
		"""Get a RestoreFile for given index, or None"""
		while 1:
			if not self.rf_list:
				if not self.add_rfs(index, mir_rorp): return None
			rf = self.rf_list[0]
			if rf.index == index:
				if Globals.process_uid != 0: self.perm_changer(index, mir_rorp)
				return rf
			elif rf.index > index:
				# Try to add earlier indicies.  But if first is
				# already from same directory, or we can't find any
				# from that directory, then we know it can't be added.
				if (index[:-1] == rf.index[:-1] or not
					self.add_rfs(index, mir_rorp)): return None
			else: del self.rf_list[0]

	def get_fp(self, index, mir_rorp):
		"""Return the file object (for reading) of given index"""
		rf = longname.update_rf(self.get_rf(index, mir_rorp), mir_rorp,
								self.root_rf.mirror_rp)
		if not rf:
			log.Log("Error: Unable to retrieve data for file %s!\nThe "
					"cause is probably data loss from the backup repository."
					% (index and "/".join(index) or '.',), 2)
			return cStringIO.StringIO('')
		return rf.get_restore_fp()

	def add_rfs(self, index, mir_rorp = None):
		"""Given index, add the rfs in that same directory

		Returns false if no rfs are available, which usually indicates
		an error.

		"""
		if not index: return self.root_rf
		if mir_rorp.has_alt_mirror_name(): return # longname alias separate
		parent_index = index[:-1]
		if Globals.process_uid != 0: self.perm_changer(parent_index)
		temp_rf = RestoreFile(self.root_rf.mirror_rp.new_index(parent_index),
							  self.root_rf.inc_rp.new_index(parent_index), [])
		new_rfs = list(temp_rf.yield_sub_rfs())
		if not new_rfs: return 0
		self.rf_list[0:0] = new_rfs
		return 1

	def close(self):
		"""Finish remaining rps in PermissionChanger"""
		if Globals.process_uid != 0: self.perm_changer.finish()


class RestoreFile:
	"""Hold data about a single mirror file and its related increments

	self.relevant_incs will be set to a list of increments that matter
	for restoring a regular file.  If the patches are to mirror_rp, it
	will be the first element in self.relevant.incs

	"""
	def __init__(self, mirror_rp, inc_rp, inc_list):
		self.index = mirror_rp.index
		self.mirror_rp = mirror_rp
		self.inc_rp, self.inc_list = inc_rp, inc_list
		self.set_relevant_incs()

	def relevant_incs_string(self):
		"""Return printable string of relevant incs, used for debugging"""
		l = ["---- Relevant incs for %s" % ("/".join(self.index),)]
		l.extend(["%s %s %s" % (inc.getinctype(), inc.lstat(), inc.path)
				  for inc in self.relevant_incs])
		l.append("--------------------------------")
		return "\n".join(l)

	def set_relevant_incs(self):
		"""Set self.relevant_incs to increments that matter for restoring

		relevant_incs is sorted newest first.  If mirror_rp matters,
		it will be (first) in relevant_incs.

		"""
		self.mirror_rp.inc_type = 'snapshot'
		self.mirror_rp.inc_compressed = 0
		if (not self.inc_list or
			MirrorStruct._rest_time >= MirrorStruct._mirror_time):
			self.relevant_incs = [self.mirror_rp]
			return

		newer_incs = self.get_newer_incs()
		i = 0
		while(i < len(newer_incs)):
			# Only diff type increments require later versions
			if newer_incs[i].getinctype() != "diff": break
			i = i+1
		self.relevant_incs = newer_incs[:i+1]
		if (not self.relevant_incs or
			self.relevant_incs[-1].getinctype() == "diff"):
			self.relevant_incs.append(self.mirror_rp)
		self.relevant_incs.reverse() # return in reversed order
		
	def get_newer_incs(self):
		"""Return list of newer incs sorted by time (increasing)

		Also discard increments older than rest_time (rest_time we are
		assuming is the exact time rdiff-backup was run, so no need to
		consider the next oldest increment or any of that)

		"""
		incpairs = []
		for inc in self.inc_list:
			time = inc.getinctime()
			if time >= MirrorStruct._rest_time: incpairs.append((time, inc))
		incpairs.sort()
		return [pair[1] for pair in incpairs]

	def get_attribs(self):
		"""Return RORP with restored attributes, but no data

		This should only be necessary if the metadata file is lost for
		some reason.  Otherwise the file provides all data.  The size
		will be wrong here, because the attribs may be taken from
		diff.

		"""
		last_inc = self.relevant_incs[-1]
		if last_inc.getinctype() == 'missing': return rpath.RORPath(self.index)

		rorp = last_inc.getRORPath()
		rorp.index = self.index
		if last_inc.getinctype() == 'dir': rorp.data['type'] = 'dir'
		return rorp

	def get_restore_fp(self):
		"""Return file object of restored data"""
		def get_fp():
			current_fp = self.get_first_fp()
			for inc_diff in self.relevant_incs[1:]:
				log.Log("Applying patch %s" % (inc_diff.get_indexpath(),), 7)
				assert inc_diff.getinctype() == 'diff'
				delta_fp = inc_diff.open("rb", inc_diff.isinccompressed())
				new_fp = tempfile.TemporaryFile()
				Rdiff.write_patched_fp(current_fp, delta_fp, new_fp)
				new_fp.seek(0)
				current_fp = new_fp
			return current_fp

		def error_handler(exc):
			log.Log("Error reading %s, substituting empty file." %
					(self.mirror_rp.path,), 2)
			return cStringIO.StringIO('')

		if not self.relevant_incs[-1].isreg():
			log.Log("""Warning: Could not restore file %s!

A regular file was indicated by the metadata, but could not be
constructed from existing increments because last increment had type
%s.  Instead of the actual file's data, an empty length file will be
created.  This error is probably caused by data loss in the
rdiff-backup destination directory, or a bug in rdiff-backup""" %
	    (self.mirror_rp.get_indexpath(), self.relevant_incs[-1].lstat()), 2)
			return cStringIO.StringIO('')
		return robust.check_common_error(error_handler, get_fp)

	def get_first_fp(self):
		"""Return first file object from relevant inc list"""
		first_inc = self.relevant_incs[0]
		assert first_inc.getinctype() == 'snapshot'
		if not first_inc.isinccompressed(): return first_inc.open("rb")

		# current_fp must be a real (uncompressed) file
		current_fp = tempfile.TemporaryFile()
		fp = first_inc.open("rb", compress = 1)
		rpath.copyfileobj(fp, current_fp)
		assert not fp.close()
		current_fp.seek(0)
		return current_fp

	def yield_sub_rfs(self):
		"""Return RestoreFiles under current RestoreFile (which is dir)"""
		if not self.mirror_rp.isdir() and not self.inc_rp.isdir(): return
		if self.mirror_rp.isdir():
			mirror_iter = self.yield_mirrorrps(self.mirror_rp)
		else: mirror_iter = iter([])
		if self.inc_rp.isdir():
			inc_pair_iter = self.yield_inc_complexes(self.inc_rp)
		else: inc_pair_iter = iter([])
		collated = rorpiter.Collate2Iters(mirror_iter, inc_pair_iter)

		for mirror_rp, inc_pair in collated:
			if not inc_pair:
				inc_rp = self.inc_rp.new_index(mirror_rp.index)
				inc_list = []
			else: inc_rp, inc_list = inc_pair
			if not mirror_rp:
				mirror_rp = self.mirror_rp.new_index_empty(inc_rp.index)
			yield self.__class__(mirror_rp, inc_rp, inc_list)

	def yield_mirrorrps(self, mirrorrp):
		"""Yield mirrorrps underneath given mirrorrp"""
		assert mirrorrp.isdir()
		for filename in robust.listrp(mirrorrp):
			rp = mirrorrp.append(filename)
			if rp.index != ('rdiff-backup-data',): yield rp

	def yield_inc_complexes(self, inc_rpath):
		"""Yield (sub_inc_rpath, inc_list) IndexedTuples from given inc_rpath

		Finds pairs under directory inc_rpath.  sub_inc_rpath will just be
		the prefix rp, while the rps in inc_list should actually exist.

		"""
		if not inc_rpath.isdir(): return

		def get_inc_pairs():
			"""Return unsorted list of (basename, inc_filenames) pairs"""
			inc_dict = {} # dictionary of basenames:inc_filenames
			dirlist = robust.listrp(inc_rpath)

			def add_to_dict(filename):
				"""Add filename to the inc tuple dictionary"""
				rp = inc_rpath.append(filename)
				if rp.isincfile() and rp.getinctype() != 'data':
					basename = rp.getincbase_str()
					inc_filename_list = inc_dict.setdefault(basename, [])
					inc_filename_list.append(filename)
				elif rp.isdir(): inc_dict.setdefault(filename, [])

			for filename in dirlist: add_to_dict(filename)
			return inc_dict.items()

		def inc_filenames2incrps(filenames):
			"""Map list of filenames into increment rps"""
			l = []
			for filename in filenames:
				rp = inc_rpath.append(filename)
				assert rp.isincfile(), rp.path
				l.append(rp)
			return l

		items = get_inc_pairs()
		items.sort() # Sorting on basis of basename now
		for (basename, inc_filenames) in items:
			sub_inc_rpath = inc_rpath.append(basename)
			yield rorpiter.IndexedTuple(sub_inc_rpath.index,
					  (sub_inc_rpath, inc_filenames2incrps(inc_filenames)))


class PatchITRB(rorpiter.ITRBranch):
	"""Patch an rpath with the given diff iters (use with IterTreeReducer)

	The main complication here involves directories.  We have to
	finish processing the directory after what's in the directory, as
	the directory may have inappropriate permissions to alter the
	contents or the dir's mtime could change as we change the
	contents.

	This code was originally taken from backup.py.  However, because
	of different error correction requirements, it is repeated here.

	"""
	def __init__(self, basis_root_rp):
		"""Set basis_root_rp, the base of the tree to be incremented"""
		self.basis_root_rp = basis_root_rp
		assert basis_root_rp.conn is Globals.local_connection
		self.dir_replacement, self.dir_update = None, None
		self.cached_rp = None

	def get_rp_from_root(self, index):
		"""Return RPath by adding index to self.basis_root_rp"""
		if not self.cached_rp or self.cached_rp.index != index:
			self.cached_rp = self.basis_root_rp.new_index(index)
		return self.cached_rp

	def can_fast_process(self, index, diff_rorp):
		"""True if diff_rorp and mirror are not directories"""
		rp = self.get_rp_from_root(index)
		return not diff_rorp.isdir() and not rp.isdir()

	def fast_process(self, index, diff_rorp):
		"""Patch base_rp with diff_rorp (case where neither is directory)"""
		rp = self.get_rp_from_root(index)
		tf = TempFile.new(rp)
		self.patch_to_temp(rp, diff_rorp, tf)
		rpath.rename(tf, rp)

	def check_hash(self, copy_report, diff_rorp):
		"""Check the hash in the copy_report with hash in diff_rorp"""
		if not diff_rorp.isreg(): return
		if not diff_rorp.has_sha1():
			log.Log("Hash for %s missing, cannot check" %
					(diff_rorp.get_indexpath()), 2)
		elif copy_report.sha1_digest == diff_rorp.get_sha1():
			log.Log("Hash %s of %s verified" %
					(diff_rorp.get_sha1(), diff_rorp.get_indexpath()), 6)
		else:
			log.Log("Warning: Hash %s of %s\ndoesn't match recorded hash %s!"
					% (copy_report.sha1_digest, diff_rorp.get_indexpath(),
					   diff_rorp.get_sha1()), 2)

	def patch_to_temp(self, basis_rp, diff_rorp, new):
		"""Patch basis_rp, writing output in new, which doesn't exist yet"""
		if diff_rorp.isflaglinked():
			Hardlink.link_rp(diff_rorp, new, self.basis_root_rp)
			return
		if diff_rorp.get_attached_filetype() == 'snapshot':
			copy_report = rpath.copy(diff_rorp, new)
		else:
			assert diff_rorp.get_attached_filetype() == 'diff'
			copy_report = Rdiff.patch_local(basis_rp, diff_rorp, new)
		self.check_hash(copy_report, diff_rorp)
		if new.lstat(): rpath.copy_attribs(diff_rorp, new)

	def start_process(self, index, diff_rorp):
		"""Start processing directory - record information for later"""
		base_rp = self.base_rp = self.get_rp_from_root(index)
		assert diff_rorp.isdir() or base_rp.isdir() or not base_rp.index
		if diff_rorp.isdir(): self.prepare_dir(diff_rorp, base_rp)
		else: self.set_dir_replacement(diff_rorp, base_rp)

	def set_dir_replacement(self, diff_rorp, base_rp):
		"""Set self.dir_replacement, which holds data until done with dir

		This is used when base_rp is a dir, and diff_rorp is not.

		"""
		assert diff_rorp.get_attached_filetype() == 'snapshot'
		self.dir_replacement = TempFile.new(base_rp)
		rpath.copy_with_attribs(diff_rorp, self.dir_replacement)
		if base_rp.isdir(): base_rp.chmod(0700)

	def prepare_dir(self, diff_rorp, base_rp):
		"""Prepare base_rp to turn into a directory"""
		self.dir_update = diff_rorp.getRORPath() # make copy in case changes
		if not base_rp.isdir():
			if base_rp.lstat(): base_rp.delete()
			base_rp.mkdir()
		base_rp.chmod(0700)

	def end_process(self):
		"""Finish processing directory"""
		if self.dir_update:
			assert self.base_rp.isdir()
			rpath.copy_attribs(self.dir_update, self.base_rp)
		else:
			assert self.dir_replacement
			self.base_rp.rmdir()
			if self.dir_replacement.lstat():
				rpath.rename(self.dir_replacement, self.base_rp)


class PermissionChanger:
	"""Change the permission of mirror files and directories

	The problem is that mirror files and directories may need their
	permissions changed in order to be read and listed, and then
	changed back when we are done.  This class hooks into the CachedRF
	object to know when an rp is needed.

	"""
	def __init__(self, root_rp):
		self.root_rp = root_rp
		self.current_index = ()
		# Below is a list of (index, rp, old_perm) triples in reverse
		# order that need clearing
		self.open_index_list = []

	def __call__(self, index, mir_rorp = None):
		"""Given rpath, change permissions up to and including index"""
		if mir_rorp and mir_rorp.has_alt_mirror_name(): return
		old_index = self.current_index
		self.current_index = index
		if not index or index <= old_index: return
		self.restore_old(index)
		self.add_new(old_index, index)

	def restore_old(self, index):
		"""Restore permissions for indicies we are done with"""
		while self.open_index_list:
			old_index, old_rp, old_perms = self.open_index_list[0]
			if index[:len(old_index)] > old_index: old_rp.chmod(old_perms)
			else: break
			del self.open_index_list[0]

	def add_new(self, old_index, index):
		"""Change permissions of directories between old_index and index"""
		for rp in self.get_new_rp_list(old_index, index):
			if ((rp.isreg() and not rp.readable()) or
				(rp.isdir() and not (rp.executable() and rp.readable()))):
				old_perms = rp.getperms()
				self.open_index_list.insert(0, (rp.index, rp, old_perms))
				if rp.isreg(): rp.chmod(0400 | old_perms)
				else: rp.chmod(0700 | old_perms)

	def get_new_rp_list(self, old_index, index):
		"""Return list of new rp's between old_index and index

		Do this lazily so that the permissions on the outer
		directories are fixed before we need the inner dirs.

		"""
		for i in range(len(index)-1, -1, -1):
			if old_index[:i] == index[:i]:
				common_prefix_len = i
				break
		else: assert 0

		for total_len in range(common_prefix_len+1, len(index)+1):
			yield self.root_rp.new_index(index[:total_len])

	def finish(self):
		"""Restore any remaining rps"""
		for index, rp, perms in self.open_index_list: rp.chmod(perms)


import Globals, Time, Rdiff, Hardlink, selection, rpath, \
	   log, robust, metadata, statistics, TempFile, hash, longname




# Copyright 2002, 2003 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""This module deal with users and groups

On each connection we may need to map unames and gnames to uids and
gids, and possibly vice-versa.  So maintain a separate dictionary for
this.

On the destination connection only, if necessary have a separate
dictionary of mappings, which specify how to map users/groups on one
connection to the users/groups on the other.  The UserMap and GroupMap
objects should only be used on the destination.

"""

try:
	import grp, pwd
except ImportError:
	pass

import log, Globals

############ "Private" section - don't use outside user_group ###########

# This should be set to the user UserMap class object if using
# user-defined user mapping, and a Map class object otherwise.
UserMap = None

# This should be set to the group UserMap class object if using
# user-defined group mapping, and a Map class object otherwise.
GroupMap = None

# Used to cache by uid2uname and gid2gname below
uid2uname_dict = {}; gid2gname_dict = {}

uname2uid_dict = {}
def uname2uid(uname):
	"""Given uname, return uid or None if cannot find"""
	try: return uname2uid_dict[uname]
	except KeyError:
		try: uid = pwd.getpwnam(uname)[2]
		except (KeyError, NameError): uid = None
		uname2uid_dict[uname] = uid
		return uid

gname2gid_dict = {}
def gname2gid(gname):
	"""Given gname, return gid or None if cannot find"""
	try: return gname2gid_dict[gname]
	except KeyError:
		try: gid = grp.getgrnam(gname)[2]
		except (KeyError, NameError): gid = None
		gname2gid_dict[gname] = gid
		return gid


class Map:
	"""Used for mapping names and id on source side to dest side"""
	def __init__(self, is_user):
		"""Initialize, user is true for users, false for groups"""
		self.name2id = (is_user and uname2uid) or gname2gid

	def __call__(self, id, name = None):
		"""Return mapped id from id and, if available, name"""
		if not name: return id
		newid = self.name2id(name)
		if newid is None: return id
		else: return newid

	def map_acl(self, id, name = None):
		"""Like get_id, but use this for ACLs.  Return id or None

		Unlike ordinary user/group ownership, ACLs are not required
		and can be dropped.  If we cannot map the name over, return
		None.

		"""
		if not name: return id
		return self.name2id(name)
		

class DefinedMap(Map):
	"""Map names and ids on source side to appropriate ids on dest side

	Like map, but initialize with user-defined mapping string, which
	supersedes Map.

	"""
	def __init__(self, is_user, mapping_string):
		"""Initialize object with given mapping string

		The mapping_string should consist of a number of lines, each which
		should have the form "source_id_or_name:dest_id_or_name".  Do user
		mapping unless user is false, then do group.

		"""
		Map.__init__(self, is_user)
		self.name_mapping_dict = {}; self.id_mapping_dict = {}

		for line in mapping_string.split('\n'):
			line = line.strip()
			if not line: continue
			comps = line.split(':')
			if not len(comps) == 2:
				log.Log.FatalError("Error parsing mapping file, bad line: "
								   + line)
			old, new = comps

			try: self.id_mapping_dict[int(old)] = self.init_get_new_id(new)
			except ValueError:
				self.name_mapping_dict[old] = self.init_get_new_id(new)

	def init_get_new_id(self, id_or_name):
		"""Return id of id_or_name, failing if cannot.  Used in __init__"""
		try: return int(id_or_name)
		except ValueError:
			try: return self.name2id(id_or_name)
			except KeyError:
				log.Log.FatalError("Cannot get id for user or group name "
								   + id_or_name)

	def __call__(self, id, name = None):
		"""Return new id given old id and name"""
		newid = self.map_acl(id, name)
		if newid is None: return id
		else: return newid

	def map_acl(self, id, name = None):
		"""Return new id or None given old and name (used for ACLs)"""
		if name:
			try: return self.name_mapping_dict[name]
			except KeyError: pass
			newid = self.name2id(name)
			if newid is not None: return newid
		try: return self.id_mapping_dict[id]
		except KeyError: return None


class NumericalMap:
	"""Simple Map replacement that just keeps numerical uid or gid"""
	def __call__(self, id, name = None): return id
	def map_acl(self, id, name = None): return id


############ Public section - can use these outside user_group ###########


def uid2uname(uid):
	"""Given uid, return uname from passwd file, or None if cannot find"""
	try: return uid2uname_dict[uid]
	except KeyError:
		try: uname = pwd.getpwuid(uid)[0]
		except (KeyError, OverflowError, NameError), e: uname = None
		uid2uname_dict[uid] = uname
		return uname

def gid2gname(gid):
	"""Given gid, return group name from group file or None if cannot find"""
	try: return gid2gname_dict[gid]
	except KeyError:
		try: gname = grp.getgrgid(gid)[0]
		except (KeyError, OverflowError, NameError), e: gname = None
		gid2gname_dict[gid] = gname
		return gname


def init_user_mapping(mapping_string = None, numerical_ids = None):
	"""Initialize user mapping with given mapping string

	If numerical_ids is set, just keep the same uid.  If either
	argument is None, default to preserving uname where possible.

	"""
	global UserMap
	if numerical_ids: UserMap = NumericalMap()
	elif mapping_string: UserMap = DefinedMap(1, mapping_string)
	else: UserMap = Map(1)

def init_group_mapping(mapping_string = None, numerical_ids = None):
	"""Initialize group mapping with given mapping string

	If numerical_ids is set, just keep the same gid.  If either
	argument is None, default to preserving gname where possible.

	"""
	global GroupMap
	if numerical_ids: GroupMap = NumericalMap()
	elif mapping_string: GroupMap = DefinedMap(0, mapping_string)
	else: GroupMap = Map(0)


def map_rpath(rp):
	"""Return mapped (newuid, newgid) from rpath's initial info

	This is the main function exported by the user_group module.  Note
	that it is connection specific.

	"""
	uid, gid = rp.getuidgid()
	uname, gname = rp.getuname(), rp.getgname()
	return (UserMap(uid, uname), GroupMap(gid, gname))

def acl_user_map(uid, uname): return UserMap.map_acl(uid, uname)
def acl_group_map(gid, gname): return GroupMap.map_acl(gid, gname)



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""MakeStatic and MakeClass

These functions are used to make all the instance methods in a class
into static or class methods.

"""

class StaticMethodsError(Exception): pass

def MakeStatic(cls):
	"""turn instance methods into static ones

	The methods (that don't begin with _) of any class that
	subclasses this will be turned into static methods.

	"""
	for name in cls.__dict__:
		if name[0] != "_":
			cls.__dict__[name] = staticmethod(cls.__dict__[name])

def MakeClass(cls):
	"""Turn instance methods into classmethods.  Ignore _ like above"""
	for name in cls.__dict__:
		if name[0] != "_":
			cls.__dict__[name] = classmethod(cls.__dict__[name])



# Copyright 2002, 2003, 2004, 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Perform various kinds of comparisons.

For instance, full-file compare, compare by hash, and metadata-only
compare.  This uses elements of the backup and restore modules.

"""

from __future__ import generators
import Globals, restore, rorpiter, log, backup, static, rpath, hash, robust

def Compare(src_rp, mirror_rp, inc_rp, compare_time):
	"""Compares metadata in src_rp dir with metadata in mirror_rp at time"""
	repo_side = mirror_rp.conn.compare.RepoSide
	data_side = src_rp.conn.compare.DataSide

	repo_iter = repo_side.init_and_get_iter(mirror_rp, inc_rp, compare_time)
	return_val = print_reports(data_side.compare_fast(repo_iter))
	repo_side.close_rf_cache()
	return return_val

def Compare_hash(src_rp, mirror_rp, inc_rp, compare_time):
	"""Compare files at src_rp with repo at compare_time

	Note metadata differences, but also check to see if file data is
	different.  If two regular files have the same size, hash the
	source and compare to the hash presumably already present in repo.

	"""
	repo_side = mirror_rp.conn.compare.RepoSide
	data_side = src_rp.conn.compare.DataSide

	repo_iter = repo_side.init_and_get_iter(mirror_rp, inc_rp, compare_time)
	return_val = print_reports(data_side.compare_hash(repo_iter))
	repo_side.close_rf_cache()
	return return_val

def Compare_full(src_rp, mirror_rp, inc_rp, compare_time):
	"""Compare full data of files at src_rp with repo at compare_time

	Like Compare_hash, but do not rely on hashes, instead copy full
	data over.

	"""
	repo_side = mirror_rp.conn.compare.RepoSide
	data_side = src_rp.conn.compare.DataSide

	src_iter = data_side.get_source_select()
	attached_repo_iter = repo_side.attach_files(src_iter, mirror_rp,
												inc_rp, compare_time)
	report_iter = data_side.compare_full(src_rp, attached_repo_iter)
	return_val = print_reports(report_iter)
	repo_side.close_rf_cache()
	return return_val

def Verify(mirror_rp, inc_rp, verify_time):
	"""Compute SHA1 sums of repository files and check against metadata"""
	assert mirror_rp.conn is Globals.local_connection
	repo_iter = RepoSide.init_and_get_iter(mirror_rp, inc_rp, verify_time)
	base_index = RepoSide.mirror_base.index

	bad_files = 0
	for repo_rorp in repo_iter:
		if not repo_rorp.isreg(): continue
		if not repo_rorp.has_sha1():
			log.Log("Warning: Cannot find SHA1 digest for file %s,\n"
					"perhaps because this feature was added in v1.1.1"
					% (repo_rorp.get_indexpath(),), 2)
			continue
		fp = RepoSide.rf_cache.get_fp(base_index + repo_rorp.index, repo_rorp)
		computed_hash = hash.compute_sha1_fp(fp)
		if computed_hash == repo_rorp.get_sha1():
			log.Log("Verified SHA1 digest of " + repo_rorp.get_indexpath(), 5)
		else:
			bad_files += 1
			log.Log("Warning: Computed SHA1 digest of %s\n   %s\n"
					"doesn't match recorded digest of\n   %s\n"
					"Your backup repository may be corrupted!" %
					(repo_rorp.get_indexpath(), computed_hash,
					 repo_rorp.get_sha1()), 2)
	RepoSide.close_rf_cache()
	if not bad_files: log.Log("Every file verified successfully.", 3)
	return bad_files

def print_reports(report_iter):
	"""Given an iter of CompareReport objects, print them to screen"""
	assert not Globals.server
	changed_files_found = 0
	for report in report_iter:
		changed_files_found = 1
		indexpath = report.index and "/".join(report.index) or "."
		print "%s: %s" % (report.reason, indexpath)

	if not changed_files_found:
		log.Log("No changes found.  Directory matches archive data.", 3)
	return changed_files_found

def get_basic_report(src_rp, repo_rorp, comp_data_func = None):
	"""Compare src_rp and repo_rorp, return CompareReport

	comp_data_func should be a function that accepts (src_rp,
	repo_rorp) as arguments, and return 1 if they have the same data,
	0 otherwise.  If comp_data_func is false, don't compare file data,
	only metadata.

	"""
	if src_rp: index = src_rp.index
	else: index = repo_rorp.index
	if not repo_rorp or not repo_rorp.lstat():
		return CompareReport(index, "new")
	elif not src_rp or not src_rp.lstat():
		return CompareReport(index, "deleted")
	elif comp_data_func and src_rp.isreg() and repo_rorp.isreg():
		if src_rp == repo_rorp: meta_changed = 0
		else: meta_changed = 1
		data_changed = comp_data_func(src_rp, repo_rorp)

		if not meta_changed and not data_changed: return None
		if meta_changed: meta_string = "metadata changed, "
		else: meta_string = "metadata the same, "
		if data_changed: data_string = "data changed"
		else: data_string = "data the same"
		return CompareReport(index, meta_string + data_string)
	elif src_rp == repo_rorp: return None
	else: return CompareReport(index, "changed")

def log_success(src_rorp, mir_rorp = None):
	"""Log that src_rorp and mir_rorp compare successfully"""
	path = src_rorp and src_rorp.get_indexpath() or mir_rorp.get_indexpath()
	log.Log("Successful compare: %s" % (path,), 5)


class RepoSide(restore.MirrorStruct):
	"""On the repository side, comparing is like restoring"""
	def init_and_get_iter(cls, mirror_rp, inc_rp, compare_time):
		"""Return rorp iter at given compare time"""
		cls.set_mirror_and_rest_times(compare_time)
		cls.initialize_rf_cache(mirror_rp, inc_rp)
		return cls.subtract_indicies(cls.mirror_base.index,
									 cls.get_mirror_rorp_iter())

	def attach_files(cls, src_iter, mirror_rp, inc_rp, compare_time):
		"""Attach data to all the files that need checking

		Return an iterator of repo rorps that includes all the files
		that may have changed, and has the fileobj set on all rorps
		that need it.

		"""
		repo_iter = cls.init_and_get_iter(mirror_rp, inc_rp, compare_time)
		base_index = cls.mirror_base.index
		for src_rorp, mir_rorp in rorpiter.Collate2Iters(src_iter, repo_iter):
			index = src_rorp and src_rorp.index or mir_rorp.index
			if src_rorp and mir_rorp:
				if not src_rorp.isreg() and src_rorp == mir_rorp:
					log_success(src_rorp, mir_rorp)
					continue # They must be equal, nothing else to check
				if (src_rorp.isreg() and mir_rorp.isreg() and
					src_rorp.getsize() == mir_rorp.getsize()):
					fp = cls.rf_cache.get_fp(base_index + index, mir_rorp)
					mir_rorp.setfile(fp)
					mir_rorp.set_attached_filetype('snapshot')

			if mir_rorp: yield mir_rorp
			else: yield rpath.RORPath(index) # indicate deleted mir_rorp

static.MakeClass(RepoSide)


class DataSide(backup.SourceStruct):
	"""On the side that has the current data, compare is like backing up"""
	def compare_fast(cls, repo_iter):
		"""Compare rorps (metadata only) quickly, return report iter"""
		src_iter = cls.get_source_select()
		for src_rorp, mir_rorp in rorpiter.Collate2Iters(src_iter, repo_iter):
			report = get_basic_report(src_rorp, mir_rorp)
			if report: yield report
			else: log_success(src_rorp, mir_rorp)

	def compare_hash(cls, repo_iter):
		"""Like above, but also compare sha1 sums of any regular files"""
		def hashes_changed(src_rp, mir_rorp):
			"""Return 0 if their data hashes same, 1 otherwise"""
			if not mir_rorp.has_sha1():
				log.Log("Warning: Metadata file has no digest for %s, "
						"unable to compare." % (mir_rorp.get_indexpath(),), 2)
				return 0
			elif (src_rp.getsize() == mir_rorp.getsize() and
				  hash.compute_sha1(src_rp) == mir_rorp.get_sha1()):
				return 0
			return 1

		src_iter = cls.get_source_select()
		for src_rp, mir_rorp in rorpiter.Collate2Iters(src_iter, repo_iter):
			report = get_basic_report(src_rp, mir_rorp, hashes_changed)
			if report: yield report
			else: log_success(src_rp, mir_rorp)

	def compare_full(cls, src_root, repo_iter):
		"""Given repo iter with full data attached, return report iter"""
		def error_handler(exc, src_rp, repo_rorp):
			log.Log("Error reading file %s" % (src_rp.path,), 2)
			return 0 # They aren't the same if we get an error

		def data_changed(src_rp, repo_rorp):
			"""Return 0 if full compare of data matches, 1 otherwise"""
			if src_rp.getsize() != repo_rorp.getsize(): return 1
			return not robust.check_common_error(error_handler,
				 rpath.cmp, (src_rp, repo_rorp))

		for repo_rorp in repo_iter:
			src_rp = src_root.new_index(repo_rorp.index)
			report = get_basic_report(src_rp, repo_rorp, data_changed)
			if report: yield report
			else: log_success(repo_rorp)
			
static.MakeClass(DataSide)


class CompareReport:
	"""When two files don't match, this tells you how they don't match

	This is necessary because the system that is doing the actual
	comparing may not be the one printing out the reports.  For speed
	the compare information can be pipelined back to the client
	connection as an iter of CompareReports.

	"""
	# self.file is added so that CompareReports can masquerate as
	# RORPaths when in an iterator, and thus get pipelined.
	file = None

	def __init__(self, index, reason):
		self.index = index
		self.reason = reason
